@@ -27,6 +27,7 @@
 #include "bits.h"
 #include "readwrite.h"
 #include "debug.h"
+#include "firmware.h"
 
 #ifdef HAVE_LUA
 #include "lua-engine.h"
@@ -295,27 +296,63 @@ public:
 
 };
 
-enum ECardMode
+enum eCardMode
 {
-	CardMode_Normal = 0,
-	CardMode_KEY1,
-	CardMode_KEY2,
+	//when the GC system is first booted up, the protocol is in raw mode
+	eCardMode_RAW = 0,
+	
+	//an intermediate stage during the protocol bootup process. commands are KEY1 encrypted.
+	eCardMode_KEY1,
+
+	//an intermediate stage during the protocol bootup process. commands are KEY1 encrypted, while replies are KEY2 encrypted
+	eCardMode_KEY2,
+
+	//the final stage of the protocol bootup process. "main data load" mode. commands are KEY2 encrypted, and replies are KEY2 encrypted.
+	//optionally, according to some flag we havent designed yet but should probably go in GCBUS_Controller, the whole KEY2 part can be bypassed
+	//(this is typical when skipping the firmware boot process)
+	eCardMode_NORMAL,
+
+	//help fix to 32bit
+	eCardMode_Pad = 0xFFFFFFFF
 };
 
-typedef struct
+//#define GCLOG(...) printf(__VA_ARGS__);
+#define GCLOG(...)
+
+void MMU_GC_endTransfer(u32 PROCNUM);
+
+struct GC_Command
 {
-	
-	u8 command[8];
+	u8 bytes[8];
+	void print()
+	{
+		GCLOG("%02X%02X%02X%02X%02X%02X%02X%02X\n",bytes[0],bytes[1],bytes[2],bytes[3],bytes[4],bytes[5],bytes[6],bytes[7]);
+	}
+	void toCryptoBuffer(u32 buf[2])
+	{
+		u8 temp[8] = { bytes[7], bytes[6], bytes[5], bytes[4], bytes[3], bytes[2], bytes[1], bytes[0] };
+		buf[0] = T1ReadLong(temp,0);
+		buf[1] = T1ReadLong(temp,4);
+	}
+	void fromCryptoBuffer(u32 buf[2])
+	{
+		bytes[7] = (buf[0]>>0)&0xFF;
+		bytes[6] = (buf[0]>>8)&0xFF;
+		bytes[5] = (buf[0]>>16)&0xFF;
+		bytes[4] = (buf[0]>>24)&0xFF;
+		bytes[3] = (buf[1]>>0)&0xFF;
+		bytes[2] = (buf[1]>>8)&0xFF;
+		bytes[1] = (buf[1]>>16)&0xFF;
+		bytes[0] = (buf[1]>>24)&0xFF;
+	}
+};
 
-	u32 address;
-	u32 transfer_count;
-
-	ECardMode mode;
-
-	// NJSD stuff
-	int blocklen;
-	
-} nds_dscard;
+//should rather be known as GCBUS controller, or somesuch
+struct GCBUS_Controller
+{
+	int transfer_count;
+	eCardMode mode; //probably only one of these
+};
 
 #define DUP2(x)  x, x
 #define DUP4(x)  x, x, x, x
@@ -369,9 +406,6 @@ struct MMU_struct
 	//32KB of shared WRAM - can be switched between ARM7 & ARM9 in two blocks
 	u8 SWIRAM[0x8000];
 
-	//Card rom & ram
-	u8 * CART_ROM;
-
 	//Unused ram
 	u8 UNUSED_RAM[4];
 
@@ -418,7 +452,7 @@ struct MMU_struct
 	u16 SPI_CNT;
 	u16 SPI_CMD;
 	u16 AUX_SPI_CNT;
-	u16 AUX_SPI_CMD;
+	//u16 AUX_SPI_CMD; //zero 20-aug-2013 - this seems pointless
 
 	u8 WRAMCNT;
 
@@ -428,9 +462,9 @@ struct MMU_struct
 	BOOL powerMan_CntRegWritten;
 	u8 powerMan_Reg[5];
 
-	memory_chip_t fw;
+	fw_memory_chip fw;
 
-	nds_dscard dscard[2];
+	GCBUS_Controller dscard[2];
 };
 
 
@@ -509,9 +543,6 @@ void MMU_DeInit(void);
 
 void MMU_Reset( void);
 
-void MMU_setRom(u8 * rom, u32 mask);
-void MMU_unsetRom( void);
-
 void print_memory_profiling( void);
 
 // Memory reading/writing (old)
@@ -654,6 +685,12 @@ FORCEINLINE u8 _MMU_read08(const int PROCNUM, const MMU_ACCESS_TYPE AT, const u3
 {
 	CheckMemoryDebugEvent(DEBUG_EVENT_READ,AT,PROCNUM,addr,8,0);
 
+	//special handling to un-protect the ARM7 bios during debug reading
+	if(PROCNUM == ARMCPU_ARM7 && AT == MMU_AT_DEBUG && addr<0x00004000)
+	{
+		return T1ReadByte(MMU.ARM7_BIOS, addr);
+	}
+
 	//special handling for DMA: read 0 from TCM
 	if(PROCNUM==ARMCPU_ARM9 && AT == MMU_AT_DMA)
 	{
@@ -683,6 +720,12 @@ FORCEINLINE u16 _MMU_read16(const int PROCNUM, const MMU_ACCESS_TYPE AT, const u
 {
 	CheckMemoryDebugEvent(DEBUG_EVENT_READ,AT,PROCNUM,addr,16,0);
 
+	//special handling to un-protect the ARM7 bios during debug reading
+	if(PROCNUM == ARMCPU_ARM7 && AT == MMU_AT_DEBUG && addr<0x00004000)
+	{
+		return T1ReadWord_guaranteedAligned(MMU.ARM7_BIOS, addr);
+	}
+
 	//special handling for DMA: read 0 from TCM
 	if(PROCNUM==ARMCPU_ARM9 && AT == MMU_AT_DMA)
 	{
@@ -725,6 +768,12 @@ FORCEINLINE u32 _MMU_read32(const int PROCNUM, const MMU_ACCESS_TYPE AT, const u
 {
 	CheckMemoryDebugEvent(DEBUG_EVENT_READ,AT,PROCNUM,addr,32,0);
 
+	//special handling to un-protect the ARM7 bios during debug reading
+	if(PROCNUM == ARMCPU_ARM7 && AT == MMU_AT_DEBUG && addr<0x00004000)
+	{
+		return T1ReadLong_guaranteedAligned(MMU.ARM7_BIOS, addr);
+	}
+
 	//special handling for DMA: read 0 from TCM
 	if(PROCNUM==ARMCPU_ARM9 && AT == MMU_AT_DMA)
 	{
