@@ -23,7 +23,7 @@
 #include "debug.h"
 #include "utils/xstring.h"
 
-#ifndef _MSC_VER
+#ifndef _MSC_VER 
 #include <stdint.h>
 #endif
 
@@ -399,7 +399,7 @@ void CHEATS::ARparser(CHEATS_LIST& list)
 					_MMU_write08<ARMCPU_ARM9,MMU_AT_DEBUG>(hi+t, tmp);
 				}
 			break;
-			default: PROGINFO("AR: ERROR uknown command 0x%2X at %08X:%08X\n", type, hi, lo); break;
+			default: PROGINFO("AR: ERROR unknown command 0x%2X at %08X:%08X\n", type, hi, lo); break;
 		}
 	}
 }
@@ -536,6 +536,22 @@ u32	CHEATS::getSize()
 	return list.size();
 }
 
+size_t CHEATS::getActiveCount()
+{
+	size_t activeCheatCount = 0;
+	const size_t cheatListCount = this->getSize();
+	
+	for (size_t i = 0; i < cheatListCount; i++)
+	{
+		if (list[i].enabled)
+		{
+			activeCheatCount++;
+		}
+	}
+	
+	return activeCheatCount;
+}
+
 void CHEATS::setDescription(const char *description, u32 pos)
 {
 	strncpy(list[pos].description, description, sizeof(list[pos].description));
@@ -653,7 +669,7 @@ BOOL CHEATS::load()
 			continue;
 		}
 		trim(buf);
-		if ((strlen(buf) == 0) || (buf[0] == ';')) continue;
+		if ((buf[0] == 0) || (buf[0] == ';')) continue;
 		if(!strncasecmp(buf,"name=",5)) continue;
 		if(!strncasecmp(buf,"serial=",7)) continue;
 
