@@ -424,7 +424,7 @@ static NSImage *iconCodeBreaker = nil;
 
 - (NSString *) memAddressString
 {
-	return [NSString stringWithFormat:@"0x%08X", self.memAddress];
+	return [NSString stringWithFormat:@"0x%08X", (unsigned int)self.memAddress];
 }
 
 - (void) setMemAddressString:(NSString *)addressString
@@ -446,7 +446,7 @@ static NSImage *iconCodeBreaker = nil;
 
 - (NSString *) memAddressSixDigitString
 {
-	return [NSString stringWithFormat:@"%06X", (self.memAddress & 0x00FFFFFF)];
+	return [NSString stringWithFormat:@"%06X", (unsigned int)(self.memAddress & 0x00FFFFFF)];
 }
 
 - (void) setMemAddressSixDigitString:(NSString *)addressString
@@ -888,6 +888,15 @@ static NSImage *iconCodeBreaker = nil;
 	return result;
 }
 
+- (NSUInteger) activeCount
+{
+	pthread_mutex_lock(self.cdsCore.mutexCoreExecute);
+	NSUInteger activeCheatsCount = self.listData->getActiveCount();
+	pthread_mutex_unlock(self.cdsCore.mutexCoreExecute);
+	
+	return activeCheatsCount;
+}
+
 - (NSMutableArray *) cheatListFromDatabase:(NSURL *)fileURL errorCode:(NSInteger *)error
 {
 	NSMutableArray *newDBList = nil;
