@@ -34,7 +34,7 @@ typedef struct
 	void *cdsController;
 	int state;
 	bool isFrameSkipEnabled;
-	unsigned int frameCount;
+	size_t frameCount;
 	unsigned int framesToSkip;
 	uint64_t timeBudgetMachAbsTime;
 	bool exitThread;
@@ -138,6 +138,10 @@ typedef struct
 - (void) removeOutput:(CocoaDSOutput *)theOutput;
 - (void) removeAllOutputs;
 
+- (NSString *) cpuEmulationEngineString;
+- (NSString *) slot1DeviceTypeString;
+- (NSString *) slot2DeviceTypeString;
+
 @end
 
 static void* RunCoreThread(void *arg);
