@@ -28,9 +28,9 @@
 #include <mach/mach.h>
 #include <mach/mach_time.h>
 
-#include "../addons.h"
 #include "../NDSSystem.h"
 #include "../slot1.h"
+#include "../slot2.h"
 #undef BOOL
 
 
@@ -99,7 +99,7 @@ static BOOL isCoreStarted = NO;
 	emuFlagDebugConsole = NO;
 	emuFlagEmulateEnsata = NO;
 	
-	slot1DeviceType = NDS_SLOT1_RETAIL;
+	slot1DeviceType = NDS_SLOT1_RETAIL_AUTO;
 	slot1StatusText = NSSTRING_STATUS_EMULATION_NOT_RUNNING;
 	
 	spinlockMasterExecute = OS_SPINLOCK_INIT;
@@ -187,7 +187,7 @@ static BOOL isCoreStarted = NO;
 		return isCoreStarted;
 	}
 	
-	addonsChangePak(NDS_ADDON_NONE);
+	slot2_Change(NDS_SLOT2_NONE);
 	
 	result = NDS_Init();
 	if (result == -1)
@@ -675,9 +675,9 @@ static BOOL isCoreStarted = NO;
 	pthread_mutex_lock(&threadParam.mutexThreadExecute);
 	
 	_slot1R4Path = (r4Path != nil) ? std::string([r4Path cStringUsingEncoding:NSUTF8StringEncoding]) : "";
-	slot1SetFatDir(_slot1R4Path);
+	slot1_SetFatDir(_slot1R4Path);
 	
-	BOOL result = slot1Change((NDS_SLOT1_TYPE)deviceTypeID);
+	BOOL result = slot1_Change((NDS_SLOT1_TYPE)deviceTypeID);
 	
 	pthread_mutex_unlock(&threadParam.mutexThreadExecute);
 	
@@ -687,7 +687,7 @@ static BOOL isCoreStarted = NO;
 			[self setSlot1StatusText:NSSTRING_STATUS_SLOT1_NO_DEVICE];
 			break;
 			
-		case NDS_SLOT1_RETAIL:
+		case NDS_SLOT1_RETAIL_AUTO:
 			[self setSlot1StatusText:NSSTRING_STATUS_SLOT1_RETAIL_INSERTED];
 			break;
 			
@@ -699,6 +699,10 @@ static BOOL isCoreStarted = NO;
 			[self setSlot1StatusText:NSSTRING_STATUS_SLOT1_R4_INSERTED];
 			break;
 			
+		case NDS_SLOT1_RETAIL_MCROM:
+			[self setSlot1StatusText:NSSTRING_STATUS_SLOT1_STANDARD_INSERTED];
+			break;
+			
 		default:
 			[self setSlot1StatusText:NSSTRING_STATUS_SLOT1_UNKNOWN_STATE];
 			break;
@@ -755,6 +759,67 @@ static BOOL isCoreStarted = NO;
 	[CocoaDSCore shutdownCore];
 }
 
+- (NSString *) cpuEmulationEngineString
+{
+	NSString *theString = @"Uninitialized";
+	
+	switch ([self cpuEmulationEngine])
+	{
+		case CPU_EMULATION_ENGINE_INTERPRETER:
+			theString = @"Interpreter";
+			break;
+			
+		case CPU_EMULATION_ENGINE_DYNAMIC_RECOMPILER:
+			theString = @"Dynamic Recompiler";
+			break;
+			
+		default:
+			break;
+	}
+	
+	return theString;
+}
+
+- (NSString *) slot1DeviceTypeString
+{
+	NSString *theString = @"Uninitialized";
+	
+	pthread_mutex_lock(&threadParam.mutexThreadExecute);
+	
+	if(slot1_device == NULL)
+	{
+		pthread_mutex_unlock(&threadParam.mutexThreadExecute);
+		return theString;
+	}
+	
+	const Slot1Info *info = slot1_device->info();
+	theString = [NSString stringWithCString:info->name() encoding:NSUTF8StringEncoding];
+	
+	pthread_mutex_unlock(&threadParam.mutexThreadExecute);
+	
+	return theString;
+}
+
+- (NSString *) slot2DeviceTypeString
+{
+	NSString *theString = @"Uninitialized";
+	
+	pthread_mutex_lock(&threadParam.mutexThreadExecute);
+	
+	if(slot2_device == NULL)
+	{
+		pthread_mutex_unlock(&threadParam.mutexThreadExecute);
+		return theString;
+	}
+	
+	const Slot2Info *info = slot2_device->info();
+	theString = [NSString stringWithCString:info->name() encoding:NSUTF8StringEncoding];
+	
+	pthread_mutex_unlock(&threadParam.mutexThreadExecute);
+	
+	return theString;
+}
+
 @end
 
 static void* RunCoreThread(void *arg)
