@@ -20,7 +20,7 @@
 #define STRING_DESMUME_SHORT_DESCRIPTION			"Nintendo DS Emulator"
 #define STRING_DESMUME_FORUM_SITE					"http://forums.desmume.org/index.php"
 #define STRING_DESMUME_TECH_SUPPORT_SITE			"http://forums.desmume.org/viewforum.php?id=4"
-#define STRING_DESMUME_BUG_REPORT_SITE				"http://sourceforge.net/tracker/?group_id=164579&atid=832291"
+#define STRING_DESMUME_BUG_REPORT_SITE				"http://sourceforge.net/p/desmume/bugs"
 
 // User Interface Localized Strings
 #define NSSTRING_TITLE_OPEN_ROM_PANEL				NSLocalizedString(@"Open ROM", nil)
@@ -46,6 +46,8 @@
 #define NSSTRING_TITLE_ENABLE_CHEATS				NSLocalizedString(@"Enable Cheats", nil)
 #define NSSTRING_TITLE_DISABLE_HUD					NSLocalizedString(@"Disable HUD", nil)
 #define NSSTRING_TITLE_ENABLE_HUD					NSLocalizedString(@"Enable HUD", nil)
+#define NSSTRING_TITLE_EXIT_FULL_SCREEN				NSLocalizedString(@"Exit Full Screen", nil)
+#define NSSTRING_TITLE_ENTER_FULL_SCREEN			NSLocalizedString(@"Enter Full Screen", nil)
 #define NSSTRING_TITLE_HIDE_STATUS_BAR				NSLocalizedString(@"Hide Status Bar", nil)
 #define NSSTRING_TITLE_SHOW_STATUS_BAR				NSLocalizedString(@"Show Status Bar", nil)
 #define NSSTRING_TITLE_HIDE_TOOLBAR					NSLocalizedString(@"Hide Toolbar", nil)
@@ -100,13 +102,16 @@
 #define NSSTRING_STATUS_EMULATION_NOT_RUNNING		NSLocalizedString(@"Emulation is not running.", nil)
 #define NSSTRING_STATUS_SLOT1_UNKNOWN_STATE			NSLocalizedString(@"Unknown state.", nil)
 #define NSSTRING_STATUS_SLOT1_NO_DEVICE				NSLocalizedString(@"No device inserted.", nil)
-#define NSSTRING_STATUS_SLOT1_RETAIL_INSERTED		NSLocalizedString(@"Retail cartridge inserted.", nil)
-#define NSSTRING_STATUS_SLOT1_RETAIL_NAND_INSERTED	NSLocalizedString(@"Retail cartridge (with NAND flash) inserted.", nil)
+#define NSSTRING_STATUS_SLOT1_RETAIL_INSERTED		NSLocalizedString(@"Retail cartridge inserted. (Auto-detect)", nil)
+#define NSSTRING_STATUS_SLOT1_RETAIL_NAND_INSERTED	NSLocalizedString(@"Retail cartridge w/ NAND flash inserted.", nil)
 #define NSSTRING_STATUS_SLOT1_R4_INSERTED			NSLocalizedString(@"R4 cartridge interface inserted.", nil)
+#define NSSTRING_STATUS_SLOT1_STANDARD_INSERTED		NSLocalizedString(@"Standard retail cartridge inserted.", nil)
+
+#define NSSTRING_STATUS_AUTOLOAD_ROM_NAME_NONE		NSLocalizedString(@"No ROM chosen.", nil)
 
 #define NSSTRING_DISPLAYMODE_MAIN					NSLocalizedString(@"Main", nil)
 #define NSSTRING_DISPLAYMODE_TOUCH					NSLocalizedString(@"Touch", nil)
-#define NSSTRING_DISPLAYMODE_COMBO					NSLocalizedString(@"Combo", nil)
+#define NSSTRING_DISPLAYMODE_DUAL					NSLocalizedString(@"Dual Screen", nil)
 
 #define NSSTRING_INPUTPREF_NUM_INPUTS_MAPPED		NSLocalizedString(@"%ld Input Mapped", nil)
 #define NSSTRING_INPUTPREF_NUM_INPUTS_MAPPED_PLURAL	NSLocalizedString(@"%ld Inputs Mapped", nil)
@@ -226,7 +231,9 @@
 enum
 {
 	ROMAUTOLOADOPTION_LOAD_LAST						= 0,
-	ROMAUTOLOADOPTION_LOAD_SELECTED					= 1
+	ROMAUTOLOADOPTION_LOAD_SELECTED					= 1,
+	ROMAUTOLOADOPTION_LOAD_NONE						= 10000,
+	ROMAUTOLOADOPTION_CHOOSE_ROM					= 10001
 };
 
 enum
@@ -419,7 +426,7 @@ enum
 {
 	DS_DISPLAY_TYPE_MAIN = 0,
 	DS_DISPLAY_TYPE_TOUCH,
-	DS_DISPLAY_TYPE_COMBO
+	DS_DISPLAY_TYPE_DUAL
 };
 
 enum
@@ -441,7 +448,7 @@ enum
 {
 	DS_GPU_TYPE_MAIN = 0,
 	DS_GPU_TYPE_SUB,
-	DS_GPU_TYPE_COMBO
+	DS_GPU_TYPE_MAIN_AND_SUB
 };
 
 /*
