@@ -37,8 +37,8 @@ typedef struct
 typedef struct
 {
 	NSInteger displayModeID;
-	unsigned int width;			// Measured in pixels
-	unsigned int height;		// Measured in pixels
+	size_t width;			// Measured in pixels
+	size_t height;			// Measured in pixels
 } DisplaySrcPixelAttributes;
 
 @interface CocoaDSOutput : CocoaDSThread
@@ -101,6 +101,9 @@ typedef struct
 - (void) setMute:(BOOL)mute;
 - (NSInteger) filter;
 - (void) setFilter:(NSInteger)filter;
+- (NSString *) audioOutputEngineString;
+- (NSString *) spuInterpolationModeString;
+- (NSString *) spuSyncMethodString;
 - (void) handleSetVolume:(NSData *)volumeData;
 - (void) handleSetAudioOutputEngine:(NSData *)methodIdData;
 - (void) handleSetSpuAdvancedLogic:(NSData *)stateData;
