@@ -264,10 +264,9 @@
 	[property setValue:[NSNumber numberWithInteger:modeID] forKey:@"spuSyncMode"];
 	OSSpinLockUnlock(&spinlockSpuSyncMode);
 	
-	NSInteger methodID = [self spuSyncMethod];
-	
 	pthread_mutex_lock(self.mutexProducer);
-	SPU_SetSynchMode(modeID, methodID);
+	CommonSettings.SPU_sync_mode = (int)modeID;
+	SPU_SetSynchMode(CommonSettings.SPU_sync_mode, CommonSettings.SPU_sync_method);
 	pthread_mutex_unlock(self.mutexProducer);
 }
 
@@ -286,10 +285,9 @@
 	[property setValue:[NSNumber numberWithInteger:methodID] forKey:@"spuSyncMethod"];
 	OSSpinLockUnlock(&spinlockSpuSyncMethod);
 	
-	NSInteger modeID = [self spuSyncMode];
-	
 	pthread_mutex_lock(self.mutexProducer);
-	SPU_SetSynchMode(modeID, methodID);
+	CommonSettings.SPU_sync_method = (int)methodID;
+	SPU_SetSynchMode(CommonSettings.SPU_sync_mode, CommonSettings.SPU_sync_method);
 	pthread_mutex_unlock(self.mutexProducer);
 }
 
@@ -331,6 +329,87 @@
 	[property setValue:[NSNumber numberWithInteger:filter] forKey:@"filter"];
 }
 
+- (NSString *) audioOutputEngineString
+{
+	NSString *theString = @"Uninitialized";
+	
+	pthread_mutex_lock(self.mutexProducer);
+	
+	SoundInterface_struct *soundCore = SPU_SoundCore();
+	if(soundCore == NULL)
+	{
+		pthread_mutex_unlock(self.mutexProducer);
+		return theString;
+	}
+	
+	const char *theName = soundCore->Name;
+	theString = [NSString stringWithCString:theName encoding:NSUTF8StringEncoding];
+	
+	pthread_mutex_unlock(self.mutexProducer);
+	
+	return theString;
+}
+
+- (NSString *) spuInterpolationModeString
+{
+	NSString *theString = @"Unknown";
+	NSInteger theMode = [self spuInterpolationMode];
+	
+	switch (theMode)
+	{
+		case SPUInterpolation_None:
+			theString = @"None";
+			break;
+			
+		case SPUInterpolation_Linear:
+			theString = @"Linear";
+			break;
+			
+		case SPUInterpolation_Cosine:
+			theString = @"Cosine";
+			break;
+			
+		default:
+			break;
+	}
+	
+	return theString;
+}
+
+- (NSString *) spuSyncMethodString
+{
+	NSString *theString = @"Unknown";
+	NSInteger theMode = [self spuSyncMode];
+	NSInteger theMethod = [self spuSyncMethod];
+	
+	if (theMode == ESynchMode_DualSynchAsynch)
+	{
+		theString = @"Dual SPU Sync/Async";
+	}
+	else
+	{
+		switch (theMethod)
+		{
+			case ESynchMethod_N:
+				theString = @"\"N\" Sync Method";
+				break;
+				
+			case ESynchMethod_Z:
+				theString = @"\"Z\" Sync Method";
+				break;
+				
+			case ESynchMethod_P:
+				theString = @"\"P\" Sync Method";
+				break;
+				
+			default:
+				break;
+		}
+	}
+	
+	return theString;
+}
+
 - (void)handlePortMessage:(NSPortMessage*)portMessage
 {
 	NSInteger message = (NSInteger)[portMessage msgid];
@@ -436,7 +515,7 @@
 	spinlockDisplayType = OS_SPINLOCK_INIT;
 	
 	delegate = nil;
-	displayMode = DS_DISPLAY_TYPE_COMBO;
+	displayMode = DS_DISPLAY_TYPE_DUAL;
 	frameSize = NSMakeSize((CGFloat)GPU_DISPLAY_WIDTH, (CGFloat)GPU_DISPLAY_HEIGHT * 2);
 	
 	[property setValue:[NSNumber numberWithInteger:displayMode] forKey:@"displayMode"];
@@ -497,8 +576,8 @@
 			newDispString = NSSTRING_DISPLAYMODE_TOUCH;
 			break;
 			
-		case DS_DISPLAY_TYPE_COMBO:
-			newDispString = NSSTRING_DISPLAYMODE_COMBO;
+		case DS_DISPLAY_TYPE_DUAL:
+			newDispString = NSSTRING_DISPLAYMODE_DUAL;
 			newFrameSize.height *= 2;
 			break;
 			
@@ -551,12 +630,12 @@
 	{
 		gpuData = [[NSData alloc] initWithBytes:(GPU_screen + GPU_SCREEN_SIZE_BYTES) length:GPU_SCREEN_SIZE_BYTES];
 	}
-	else if(displayModeID == DS_DISPLAY_TYPE_COMBO)
+	else if(displayModeID == DS_DISPLAY_TYPE_DUAL)
 	{
 		gpuData = [[NSData alloc] initWithBytes:GPU_screen length:GPU_SCREEN_SIZE_BYTES * 2];
 	}
 	
-	DisplaySrcPixelAttributes attr = {displayModeID, (unsigned int)displayFrameSize.width, (unsigned int)displayFrameSize.height};
+	DisplaySrcPixelAttributes attr = {displayModeID, (size_t)displayFrameSize.width, (size_t)displayFrameSize.height};
 	NSData *attributesData = [[NSData alloc] initWithBytes:&attr length:sizeof(DisplaySrcPixelAttributes)];
 	
 	NSArray *messageComponents = [[NSArray alloc] initWithObjects:gpuData, attributesData, nil];
@@ -665,15 +744,9 @@
 
 - (void) fillVideoFrameWithColor:(UInt16)colorValue
 {
-	NSData *gpuData = nil;
-	NSInteger displayModeID = [self displayMode];
-	NSSize displayFrameSize = [self frameSize];
-	size_t numberBytes = GPU_SCREEN_SIZE_BYTES * 2;
-	
-	if (displayModeID == DS_DISPLAY_TYPE_MAIN || displayModeID == DS_DISPLAY_TYPE_TOUCH)
-	{
-		numberBytes = GPU_SCREEN_SIZE_BYTES;
-	}
+	const NSInteger displayModeID = [self displayMode];
+	const NSSize displayFrameSize = [self frameSize];
+	const size_t numberBytes = (displayModeID == DS_DISPLAY_TYPE_DUAL) ? GPU_SCREEN_SIZE_BYTES * 2 : GPU_SCREEN_SIZE_BYTES;
 	
 	UInt16 *gpuBytes = (UInt16 *)malloc(numberBytes);
 	if (gpuBytes == NULL)
@@ -683,15 +756,16 @@
 	
 	const UInt16 colorValuePattern[] = {colorValue, colorValue, colorValue, colorValue, colorValue, colorValue, colorValue, colorValue};
 	memset_pattern16(gpuBytes, colorValuePattern, numberBytes);
-	gpuData = [[[NSData alloc] initWithBytes:gpuBytes length:numberBytes] autorelease];
+	NSData *gpuData = [[NSData alloc] initWithBytes:gpuBytes length:numberBytes];
 	
 	free(gpuBytes);
 	gpuBytes = nil;
 	
-	DisplaySrcPixelAttributes attr = {displayModeID, (unsigned int)displayFrameSize.width, (unsigned int)displayFrameSize.height};
+	DisplaySrcPixelAttributes attr = {displayModeID, (size_t)displayFrameSize.width, (size_t)displayFrameSize.height};
 	NSData *attributesData = [[[NSData alloc] initWithBytes:&attr length:sizeof(DisplaySrcPixelAttributes)] autorelease];
 	
 	[self handleEmuFrameProcessed:gpuData attributes:attributesData];
+	[gpuData release];
 }
 
 - (NSImage *) image
@@ -772,7 +846,7 @@
 	}
 	
 	videoDelegate = nil;
-	lastDisplayMode = DS_DISPLAY_TYPE_COMBO;
+	lastDisplayMode = DS_DISPLAY_TYPE_DUAL;
 	
 	spinlockVideoFilterType = OS_SPINLOCK_INIT;
 	spinlockVFBuffers = OS_SPINLOCK_INIT;
