@@ -39,11 +39,13 @@
 @property (readonly) NSMutableDictionary *header;
 @property (readonly) NSMutableDictionary *bindings;
 @property (readonly) NSURL *fileURL;
+@property (assign) BOOL willStreamLoadData;
 @property (readonly) BOOL isDataLoaded;
 @property (assign) NSInteger saveType;
 
 - (id) initWithURL:(NSURL *)theURL;
 - (id) initWithURL:(NSURL *)theURL saveType:(NSInteger)saveTypeID;
+- (id) initWithURL:(NSURL *)theURL saveType:(NSInteger)saveTypeID streamLoadData:(BOOL)willStreamLoad;
 - (void) initHeader;
 - (BOOL) loadData:(NSURL *)theURL;
 - (void) loadDataOnThread:(id)object;
@@ -52,6 +54,8 @@
 - (NSString *) banner:(const UInt16 *)UTF16TextBuffer;
 - (NSString *) internalName;
 - (NSString *) serial;
+- (NSString *) developerName;
+- (NSString *) developerNameAndCode;
 - (NSImage *) icon;
 - (void) handleAdvansceneDatabaseInfo;
 
