@@ -32,6 +32,7 @@
 @synthesize header;
 @synthesize bindings;
 @synthesize fileURL;
+@dynamic willStreamLoadData;
 @dynamic isDataLoaded;
 @synthesize saveType;
 
@@ -48,6 +49,11 @@ static NSMutableDictionary *saveTypeValues = nil;
 }
 
 - (id) initWithURL:(NSURL *)theURL saveType:(NSInteger)saveTypeID
+{
+	return [self initWithURL:theURL saveType:ROMSAVETYPE_AUTOMATIC streamLoadData:NO];
+}
+
+- (id) initWithURL:(NSURL *)theURL saveType:(NSInteger)saveTypeID streamLoadData:(BOOL)willStreamLoad
 {
 	self = [super init];
 	if (self == nil)
@@ -94,6 +100,8 @@ static NSMutableDictionary *saveTypeValues = nil;
 	xmlElementStack = [[NSMutableArray alloc] initWithCapacity:32];
 	xmlCharacterStack = [[NSMutableArray alloc] initWithCapacity:32];
 	
+	[self setWillStreamLoadData:willStreamLoad];
+	
 	if (theURL != nil)
 	{
 		[self loadData:theURL];
@@ -118,9 +126,19 @@ static NSMutableDictionary *saveTypeValues = nil;
 	[super dealloc];
 }
 
+- (void) setWillStreamLoadData:(BOOL)theState
+{
+	CommonSettings.loadToMemory = (theState) ? false : true;
+}
+
+- (BOOL) willStreamLoadData
+{
+	return (CommonSettings.loadToMemory ? NO : YES);
+}
+
 - (BOOL) isDataLoaded
 {
-	return (MMU.CART_ROM != MMU.UNUSED_RAM);
+	return (gameInfo.romdata != NULL);
 }
 
 - (void) initHeader
@@ -154,6 +172,8 @@ static NSMutableDictionary *saveTypeValues = nil;
 	{
 		[self.header setValue:[self title] forKey:@"gameTitle"];
 		[self.header setValue:[self code] forKey:@"gameCode"];
+		[self.header setValue:[self developerName] forKey:@"gameDeveloper"];
+		[self.header setValue:[self developerNameAndCode] forKey:@"gameDeveloperWithCode"];
 		[self.header setValue:[NSNumber numberWithInteger:ndsRomHeader->makerCode] forKey:@"makerCode"];
 		[self.header setValue:[NSNumber numberWithInteger:ndsRomHeader->cardSize] forKey:@"romSize"];
 		[self.header setValue:[NSNumber numberWithInteger:ndsRomHeader->ARM9src] forKey:@"arm9BinaryOffset"];
@@ -169,11 +189,13 @@ static NSMutableDictionary *saveTypeValues = nil;
 		[self.header setValue:[NSNumber numberWithInteger:ndsRomHeader->FATOff] forKey:@"fatOffset"];
 		[self.header setValue:[NSNumber numberWithInteger:ndsRomHeader->FATSize] forKey:@"fatSize"];
 		[self.header setValue:[NSNumber numberWithInteger:ndsRomHeader->IconOff] forKey:@"iconOffset"];
-		[self.header setValue:[NSNumber numberWithInteger:ndsRomHeader->ROMSize] forKey:@"usedRomSize"];
+		[self.header setValue:[NSNumber numberWithInteger:ndsRomHeader->endROMoffset] forKey:@"usedRomSize"];
 		
 		[self.bindings setValue:[self.header objectForKey:@"gameTitle"] forKey:@"gameTitle"];
 		[self.bindings setValue:[self.header objectForKey:@"gameCode"] forKey:@"gameCode"];
-		[self.bindings setValue:[NSString stringWithFormat:@"%04Xh", [[self.header objectForKey:@"makerCode"] intValue]] forKey:@"makerCode"];
+		[self.bindings setValue:[self.header objectForKey:@"gameDeveloper"] forKey:@"gameDeveloper"];
+		[self.bindings setValue:[self.header objectForKey:@"gameDeveloperWithCode"] forKey:@"gameDeveloperWithCode"];
+		[self.bindings setValue:[NSString stringWithFormat:@"0x%04X", [[self.header objectForKey:@"makerCode"] intValue]] forKey:@"makerCode"];
 		[self.bindings setValue:[NSString stringWithFormat:@"%i", [[self.header objectForKey:@"romSize"] intValue]] forKey:@"romSize"];
 		[self.bindings setValue:[NSString stringWithFormat:@"0x%08X", [[self.header objectForKey:@"arm9BinaryOffset"] intValue]] forKey:@"arm9BinaryOffset"];
 		[self.bindings setValue:[NSString stringWithFormat:@"0x%08X", [[self.header objectForKey:@"arm9BinaryEntryAddress"] intValue]] forKey:@"arm9BinaryEntryAddress"];
@@ -300,6 +322,28 @@ static NSMutableDictionary *saveTypeValues = nil;
 	return [NSString stringWithCString:gameInfo.ROMserial encoding:NSUTF8StringEncoding];
 }
 
+- (NSString *) developerName
+{
+	NDS_header *ndsRomHeader = NDS_getROMHeader();
+	if (ndsRomHeader == nil)
+	{
+		return nil;
+	}
+	
+	return [NSString stringWithCString:getDeveloperNameByID(ndsRomHeader->makerCode).c_str() encoding:NSUTF8StringEncoding];
+}
+
+- (NSString *) developerNameAndCode
+{
+	NDS_header *ndsRomHeader = NDS_getROMHeader();
+	if (ndsRomHeader == nil)
+	{
+		return nil;
+	}
+	
+	return [NSString stringWithFormat:@"%s [0x%04X]", getDeveloperNameByID(ndsRomHeader->makerCode).c_str(), ndsRomHeader->makerCode];
+}
+
 - (NSImage *) icon
 {
 	NSImage *newImage = nil;
@@ -501,6 +545,8 @@ static NSMutableDictionary *saveTypeValues = nil;
 			NSSTRING_STATUS_NO_ROM_LOADED, @"bannerSpanish",
 			NSSTRING_STATUS_NO_ROM_LOADED, @"gameTitle",
 			NSSTRING_STATUS_NO_ROM_LOADED, @"gameCode",
+			NSSTRING_STATUS_NO_ROM_LOADED, @"gameDeveloper",
+			NSSTRING_STATUS_NO_ROM_LOADED, @"gameDeveloperWithCode",
 			NSSTRING_STATUS_NO_ROM_LOADED, @"makerCode",
 			[NSString stringWithFormat:NSSTRING_STATUS_NO_ROM_LOADED, 0], @"romSize",
 			[NSString stringWithFormat:@"0x%08X", 0], @"arm9BinaryOffset",
@@ -544,8 +590,7 @@ static NSMutableDictionary *saveTypeValues = nil;
  ********************************************************************************************/
 void RomIconToRGBA8888(uint32_t *bitmapData)
 {
-	const NDS_header *ndsRomHeader = NDS_getROMHeader();	// Contains the memory addresses we need to get our read pointer locations.
-	int iconOffset;					// Memory location offset for the ROM icon data.
+	const RomBanner &ndsRomBanner = gameInfo.getRomBanner(); // Contains the memory addresses we need to get our read pointer locations.
 	const uint16_t *iconClutPtr;	// Read pointer for the icon's CLUT.
 	const uint32_t *iconPixPtr;		// Read pointer for the icon's pixel data.
 	
@@ -563,16 +608,15 @@ void RomIconToRGBA8888(uint32_t *bitmapData)
 		return;
 	}
 	
-	if (ndsRomHeader == NULL)
+	if (&ndsRomBanner == NULL)
 	{
 		memset(bitmapData, 0, 4096); // 4096 bytes = 32px * 32px * sizeof(uint32_t)
 		return;
 	}
 	
-	// Set all of our pointers relative to iconOffset.
-	iconOffset = ndsRomHeader->IconOff;
-	iconClutPtr = (uint16_t *)&MMU.CART_ROM[iconOffset + 0x220] + 1;
-	iconPixPtr = (uint32_t *)&MMU.CART_ROM[iconOffset + 0x20];
+	// Set all of our icon read pointers.
+	iconClutPtr = (uint16_t *)ndsRomBanner.palette + 1;
+	iconPixPtr = (uint32_t *)ndsRomBanner.bitmap;
 	
 	// Setup the 4-bit CLUT.
 	//
