@@ -61,6 +61,7 @@
 
 @dynamic masterExecuteFlag;
 @dynamic executionState;
+@synthesize lastSetSpeedScalar;
 @dynamic speedScalar;
 
 @synthesize isWorking;
@@ -430,13 +431,6 @@
 - (IBAction) newDisplayWindow:(id)sender
 {
 	DisplayWindowController *newWindowController = [[DisplayWindowController alloc] initWithWindowNibName:@"DisplayWindow" emuControlDelegate:self];
-	[windowList addObject:newWindowController];
-	[[newWindowController view] setInputManager:[self inputManager]];
-	
-	[self updateAllWindowTitles];
-	[newWindowController showWindow:self];
-	[[newWindowController window] makeKeyAndOrderFront:self];
-	[[newWindowController window] makeMainWindow];
 	
 	if ([self currentRom] == nil)
 	{
@@ -446,6 +440,9 @@
 	{
 		[[newWindowController view] setNeedsDisplay:YES];
 	}
+	
+	[[newWindowController window] makeKeyAndOrderFront:self];
+	[[newWindowController window] makeMainWindow];
 }
 
 - (IBAction) openRom:(id)sender
@@ -482,7 +479,7 @@
 			return;
 		}
 		
-		[self handleLoadRom:selectedFile];
+		[self handleLoadRomByURL:selectedFile];
 	}
 }
 
@@ -491,11 +488,6 @@
 	// Dummy selector, used for UI validation only.
 }
 
-- (IBAction) closeWindow:(id)sender
-{
-	[[mainWindow window] performClose:sender];
-}
-
 - (IBAction) closeRom:(id)sender
 {
 	[self handleUnloadRom:REASONFORCLOSE_NORMAL romToLoad:nil];
@@ -756,31 +748,6 @@
 	[inputManager dispatchCommandUsingIBAction:_cmd sender:sender];
 }
 
-- (IBAction) toggleKeepMinDisplaySizeAtNormal:(id)sender
-{
-	[mainWindow toggleKeepMinDisplaySizeAtNormal:sender];
-}
-
-- (IBAction) toggleStatusBar:(id)sender
-{
-	[mainWindow toggleStatusBar:sender];
-}
-
-- (IBAction) toggleToolbarShown:(id)sender
-{
-	[[mainWindow window] toggleToolbarShown:sender];
-}
-
-- (IBAction) runToolbarCustomizationPalette:(id)sender
-{
-	[[mainWindow window] runToolbarCustomizationPalette:sender];
-}
-
-- (IBAction) saveScreenshotAs:(id)sender
-{
-	[mainWindow saveScreenshotAs:sender];
-}
-
 - (IBAction) toggleGPUState:(id)sender
 {
 	[inputManager dispatchCommandUsingIBAction:_cmd sender:sender];
@@ -802,10 +769,15 @@
 
 - (IBAction) changeCoreSpeed:(id)sender
 {
-	CGFloat newSpeedScalar = (CGFloat)[CocoaDSUtil getIBActionSenderTag:sender] / 100.0f;
-	CocoaDSCore *cdsCore = (CocoaDSCore *)[cdsCoreController content];
-	[cdsCore setSpeedScalar:newSpeedScalar];
-	lastSetSpeedScalar = newSpeedScalar;
+	if ([sender isKindOfClass:[NSSlider class]])
+	{
+		lastSetSpeedScalar = [(NSSlider *)sender floatValue];
+	}
+	else
+	{
+		const CGFloat newSpeedScalar = (CGFloat)[CocoaDSUtil getIBActionSenderTag:sender] / 100.0f;
+		[self changeCoreSpeedWithDouble:newSpeedScalar];
+	}
 }
 
 - (IBAction) changeCoreEmuFlags:(id)sender
@@ -874,63 +846,6 @@
 	[CocoaDSUtil messageSendOneWayWithInteger:[cdsSpeaker receivePort] msgID:MESSAGE_SET_SPU_SYNC_METHOD integerValue:[CocoaDSUtil getIBActionSenderTag:sender]];
 }
 
-- (IBAction) changeScale:(id)sender
-{
-	[mainWindow setDisplayScale:(double)[CocoaDSUtil getIBActionSenderTag:sender] / 100.0];
-}
-
-- (IBAction) changeRotation:(id)sender
-{
-	// Get the rotation value from the sender.
-	if ([sender isMemberOfClass:[NSSlider class]])
-	{
-		[mainWindow setDisplayRotation:[(NSSlider *)sender doubleValue]];
-	}
-	else
-	{
-		[mainWindow setDisplayRotation:(double)[CocoaDSUtil getIBActionSenderTag:sender]];
-	}
-}
-
-- (IBAction) changeRotationRelative:(id)sender
-{
-	[inputManager dispatchCommandUsingIBAction:_cmd sender:sender];
-}
-
-- (IBAction) changeDisplayMode:(id)sender
-{
-	const NSInteger newDisplayModeID = [CocoaDSUtil getIBActionSenderTag:sender];
-	
-	if (newDisplayModeID == [mainWindow displayMode])
-	{
-		return;
-	}
-	
-	[mainWindow setDisplayMode:newDisplayModeID];
-}
-
-- (IBAction) changeDisplayOrientation:(id)sender
-{
-	const NSInteger newDisplayOrientation = [CocoaDSUtil getIBActionSenderTag:sender];
-	
-	if (newDisplayOrientation == [mainWindow displayOrientation])
-	{
-		return;
-	}
-	
-	[mainWindow setDisplayOrientation:newDisplayOrientation];
-}
-
-- (IBAction) changeDisplayOrder:(id)sender
-{
-	[mainWindow setDisplayOrder:[CocoaDSUtil getIBActionSenderTag:sender]];
-}
-
-- (IBAction) changeDisplayGap:(id)sender
-{
-	[mainWindow setDisplayGap:(double)[CocoaDSUtil getIBActionSenderTag:sender] / 100.0];
-}
-
 - (IBAction) chooseSlot1R4Directory:(id)sender
 {
 	NSOpenPanel *panel = [NSOpenPanel openPanel];
@@ -939,12 +854,10 @@
 	[panel setResolvesAliases:YES];
 	[panel setAllowsMultipleSelection:NO];
 	[panel setTitle:@"Select R4 Directory"];
-	NSArray *fileTypes = [NSArray arrayWithObjects:nil];
 	
 	// The NSOpenPanel/NSSavePanel method -(void)beginSheetForDirectory:file:types:modalForWindow:modalDelegate:didEndSelector:contextInfo
 	// is deprecated in Mac OS X v10.6.
 #if MAC_OS_X_VERSION_MIN_REQUIRED > MAC_OS_X_VERSION_10_5
-	[panel setAllowedFileTypes:fileTypes];
 	[panel beginSheetModalForWindow:slot1ManagerWindow
 				  completionHandler:^(NSInteger result) {
 					  [self didEndChooseSlot1R4Directory:panel returnCode:result contextInfo:nil];
@@ -952,7 +865,7 @@
 #else
 	[panel beginSheetForDirectory:nil
 							 file:nil
-							types:fileTypes
+							types:nil
 				   modalForWindow:slot1ManagerWindow
 					modalDelegate:self
 				   didEndSelector:@selector(didEndChooseSlot1R4Directory:returnCode:contextInfo:)
@@ -966,28 +879,6 @@
 	[cdsCore slot1Eject];
 }
 
-- (IBAction) writeDefaultsDisplayRotation:(id)sender
-{
-	[[NSUserDefaults standardUserDefaults] setDouble:[mainWindow displayRotation] forKey:@"DisplayView_Rotation"];
-}
-
-- (IBAction) writeDefaultsDisplayGap:(id)sender
-{
-	[[NSUserDefaults standardUserDefaults] setDouble:([mainWindow displayGap] * 100.0) forKey:@"DisplayViewCombo_Gap"];
-}
-
-- (IBAction) writeDefaultsHUDSettings:(id)sender
-{
-	// TODO: Not implemented.
-}
-
-- (IBAction) writeDefaultsDisplayVideoSettings:(id)sender
-{
-	[[NSUserDefaults standardUserDefaults] setInteger:[mainWindow videoFilterType] forKey:@"DisplayView_VideoFilter"];
-	[[NSUserDefaults standardUserDefaults] setBool:[mainWindow useBilinearOutput] forKey:@"DisplayView_UseBilinearOutput"];
-	[[NSUserDefaults standardUserDefaults] setBool:[mainWindow useVerticalSync] forKey:@"DisplayView_UseVerticalSync"];
-}
-
 - (IBAction) writeDefaults3DRenderingSettings:(id)sender
 {
 	CocoaDSCore *cdsCore = (CocoaDSCore *)[cdsCoreController content];
@@ -1394,11 +1285,11 @@
 	[cdsCore.cdsGPU setGpuStateFlags:flagBit];
 }
 
-- (BOOL) handleLoadRom:(NSURL *)fileURL
+- (BOOL) handleLoadRomByURL:(NSURL *)fileURL
 {
 	BOOL result = NO;
 	
-	if ([self isRomLoading])
+	if (fileURL == nil || [self isRomLoading])
 	{
 		return result;
 	}
@@ -1432,7 +1323,7 @@
 	}
 	else
 	{
-		result = [self loadRom:fileURL];
+		result = [self loadRomByURL:fileURL asynchronous:YES];
 	}
 	
 	return result;
@@ -1485,7 +1376,7 @@
 	return result;
 }
 
-- (BOOL) loadRom:(NSURL *)romURL
+- (BOOL) loadRomByURL:(NSURL *)romURL asynchronous:(BOOL)willLoadAsync
 {
 	BOOL result = NO;
 	
@@ -1516,7 +1407,17 @@
 		[self setIsRomLoading:YES];
 		[romURL retain];
 		[newRom setSaveType:selectedRomSaveTypeID];
-		[NSThread detachNewThreadSelector:@selector(loadDataOnThread:) toTarget:newRom withObject:romURL];
+		[newRom setWillStreamLoadData:[[NSUserDefaults standardUserDefaults] boolForKey:@"General_StreamLoadRomData"]];
+		
+		if (willLoadAsync)
+		{
+			[NSThread detachNewThreadSelector:@selector(loadDataOnThread:) toTarget:newRom withObject:romURL];
+		}
+		else
+		{
+			[newRom loadData:romURL];
+		}
+		
 		[romURL release];
 	}
 	
@@ -1531,7 +1432,7 @@
 	NSDictionary *userInfo = [aNotification userInfo];
 	const BOOL didLoad = [(NSNumber *)[userInfo valueForKey:@"DidLoad"] boolValue];
 	
-	if (theRom == nil || ![theRom isDataLoaded] || !didLoad)
+	if ( theRom == nil || !didLoad || (![theRom willStreamLoadData] && ![theRom isDataLoaded]) )
 	{
 		// If ROM loading fails, restore the core state, but only if a ROM is already loaded.
 		if([self currentRom] != nil)
@@ -1733,6 +1634,13 @@
 	[cdsCore removeOutput:theOutput];
 }
 
+- (void) changeCoreSpeedWithDouble:(double)newSpeedScalar
+{
+	CocoaDSCore *cdsCore = (CocoaDSCore *)[cdsCoreController content];
+	[cdsCore setSpeedScalar:newSpeedScalar];
+	lastSetSpeedScalar = newSpeedScalar;
+}
+
 - (void) executeCore
 {
 	CocoaDSCore *cdsCore = (CocoaDSCore *)[cdsCoreController content];
@@ -1782,7 +1690,7 @@
 	
 	[self setIsUserInterfaceBlockingExecution:NO];
 	[self setIsShowingFileMigrationDialog:NO];
-	[self loadRom:romURL];
+	[self loadRomByURL:romURL asynchronous:YES];
 	
 	// We retained this when we initially put up the sheet, so we need to release it now.
 	[romURL release];
@@ -1830,7 +1738,7 @@
 	[self didEndSaveStateSheet:sheet returnCode:returnCode contextInfo:contextInfo];
 	
 	NSURL *romURL = (NSURL *)contextInfo;
-	[self handleLoadRom:romURL];
+	[self handleLoadRomByURL:romURL];
 	[romURL release];
 }
 
@@ -1900,14 +1808,14 @@
 		{
 			NSString *newWindowTitle = [romName stringByAppendingFormat:@":%ld", (unsigned long)([windowList indexOfObject:windowController] + 1)];
 			
-			[[windowController window] setTitle:newWindowTitle];
-			[[windowController window] setRepresentedURL:repURL];
-			[[[windowController window] standardWindowButton:NSWindowDocumentIconButton] setImage:titleIcon];
+			[[windowController masterWindow] setTitle:newWindowTitle];
+			[[windowController masterWindow] setRepresentedURL:repURL];
+			[[[windowController masterWindow] standardWindowButton:NSWindowDocumentIconButton] setImage:titleIcon];
 		}
 	}
 	else
 	{
-		NSWindow *theWindow = [[windowList objectAtIndex:0] window];
+		NSWindow *theWindow = [[windowList objectAtIndex:0] masterWindow];
 		[theWindow setTitle:romName];
 		[theWindow setRepresentedURL:repURL];
 		[[theWindow standardWindowButton:NSWindowDocumentIconButton] setImage:titleIcon];
@@ -2151,120 +2059,6 @@
 			[(NSMenuItem*)theItem setState:([cdsCore.cdsGPU gpuStateByBit:[theItem tag]]) ? NSOnState : NSOffState];
 		}
 	}
-	else if (theAction == @selector(changeScale:))
-	{
-		const NSInteger viewScale = (NSInteger)([mainWindow displayScale] * 100.0);
-		
-		if ([(id)theItem isMemberOfClass:[NSMenuItem class]])
-		{
-			[(NSMenuItem*)theItem setState:(viewScale == [theItem tag]) ? NSOnState : NSOffState];
-		}
-	}
-	else if (theAction == @selector(changeRotation:))
-	{
-		const NSInteger viewRotation = (NSInteger)[mainWindow displayRotation];
-		
-		if ([(id)theItem isMemberOfClass:[NSMenuItem class]])
-		{
-			if ([theItem tag] == -1)
-			{
-				if (viewRotation == 0 ||
-					viewRotation == 90 ||
-					viewRotation == 180 ||
-					viewRotation == 270)
-				{
-					[(NSMenuItem*)theItem setState:NSOffState];
-				}
-				else
-				{
-					[(NSMenuItem*)theItem setState:NSOnState];
-				}
-			}
-			else
-			{
-				[(NSMenuItem*)theItem setState:(viewRotation == [theItem tag]) ? NSOnState : NSOffState];
-			}
-		}
-	}
-	else if (theAction == @selector(changeDisplayMode:))
-	{
-		if ([(id)theItem isMemberOfClass:[NSMenuItem class]])
-		{
-			[(NSMenuItem*)theItem setState:([mainWindow displayMode] == [theItem tag]) ? NSOnState : NSOffState];
-		}
-	}
-	else if (theAction == @selector(changeDisplayOrientation:))
-	{
-		if ([(id)theItem isMemberOfClass:[NSMenuItem class]])
-		{
-			[(NSMenuItem*)theItem setState:([mainWindow displayOrientation] == [theItem tag]) ? NSOnState : NSOffState];
-		}
-	}
-	else if (theAction == @selector(changeDisplayOrder:))
-	{
-		if ([(id)theItem isMemberOfClass:[NSMenuItem class]])
-		{
-			[(NSMenuItem*)theItem setState:([mainWindow displayOrder] == [theItem tag]) ? NSOnState : NSOffState];
-		}
-	}
-	else if (theAction == @selector(changeDisplayGap:))
-	{
-		if ([(id)theItem isMemberOfClass:[NSMenuItem class]])
-		{
-			const NSInteger gapScalar = (NSInteger)([mainWindow displayGap] * 100.0);
-			
-			if ([(id)theItem isMemberOfClass:[NSMenuItem class]])
-			{
-				if ([theItem tag] == -1)
-				{
-					if (gapScalar == 0 ||
-						gapScalar == 50 ||
-						gapScalar == 100 ||
-						gapScalar == 150 ||
-						gapScalar == 200)
-					{
-						[(NSMenuItem*)theItem setState:NSOffState];
-					}
-					else
-					{
-						[(NSMenuItem*)theItem setState:NSOnState];
-					}
-				}
-				else
-				{
-					[(NSMenuItem*)theItem setState:(gapScalar == [theItem tag]) ? NSOnState : NSOffState];
-				}
-			}
-		}
-	}
-	else if (theAction == @selector(hudDisable:))
-	{
-		if ([(id)theItem isMemberOfClass:[NSMenuItem class]])
-		{
-			[(NSMenuItem*)theItem setTitle:([[mainWindow view] isHudEnabled]) ? NSSTRING_TITLE_DISABLE_HUD : NSSTRING_TITLE_ENABLE_HUD];
-		}
-	}
-	else if (theAction == @selector(toggleStatusBar:))
-	{
-		if ([(id)theItem isMemberOfClass:[NSMenuItem class]])
-		{
-			[(NSMenuItem*)theItem setTitle:([mainWindow isShowingStatusBar]) ? NSSTRING_TITLE_HIDE_STATUS_BAR : NSSTRING_TITLE_SHOW_STATUS_BAR];
-		}
-	}
-	else if (theAction == @selector(toggleKeepMinDisplaySizeAtNormal:))
-	{
-		if ([(id)theItem isMemberOfClass:[NSMenuItem class]])
-		{
-			[(NSMenuItem*)theItem setState:([mainWindow isMinSizeNormal]) ? NSOnState : NSOffState];
-		}
-	}
-	else if (theAction == @selector(toggleToolbarShown:))
-	{
-		if ([(id)theItem isMemberOfClass:[NSMenuItem class]])
-		{
-			[(NSMenuItem*)theItem setTitle:([[[mainWindow window] toolbar] isVisible]) ? NSSTRING_TITLE_HIDE_TOOLBAR : NSSTRING_TITLE_SHOW_TOOLBAR];
-		}
-	}
 	
 	return enable;
 }
