@@ -201,9 +201,9 @@ static NSDictionary *hidUsageTable = nil;
 
 	Takes:
 		hidValueRef - The IOHIDValueRef to parse.
-		altElementCode - An NSString that overrides the default element code.
-		altElementName - An NSString that overrides the default element name.
-		altOnState - An NSNumber that overrides the default on state.
+		altElementCode - A char buffer that overrides the default element code.
+		altElementName - A char buffer that overrides the default element name.
+		altOnState - A pointer to a bool value that overrides the default on state.
 
 	Returns:
 		An InputAttributes struct with the parsed input attributes.
@@ -211,7 +211,7 @@ static NSDictionary *hidUsageTable = nil;
 	Details:
 		None.
  ********************************************************************************************/
-InputAttributes InputAttributesOfHIDValue(IOHIDValueRef hidValueRef, const char *altElementCode, const char *altElementName, NSNumber *altOnState)
+InputAttributes InputAttributesOfHIDValue(IOHIDValueRef hidValueRef, const char *altElementCode, const char *altElementName, bool *altOnState)
 {
 	InputAttributes inputAttr;
 	
@@ -303,7 +303,7 @@ InputAttributes InputAttributesOfHIDValue(IOHIDValueRef hidValueRef, const char
 		CFStringGetCString(cfDeviceName, inputAttr.deviceName, INPUT_HANDLER_STRING_LENGTH, kCFStringEncodingUTF8);
 	}
 	
-	BOOL onState = (altOnState == nil) ? GetOnStateFromHIDValueRef(hidValueRef) : [altOnState boolValue];
+	bool onState = (altOnState == NULL) ? GetOnStateFromHIDValueRef(hidValueRef) : *altOnState;
 	CFIndex logicalValue = IOHIDValueGetIntegerValue(hidValueRef);
 	NSInteger logicalMin = IOHIDElementGetLogicalMin(hidElementRef);
 	NSInteger logicalMax = IOHIDElementGetLogicalMax(hidElementRef);
@@ -348,14 +348,14 @@ InputAttributesList InputListFromHIDValue(IOHIDValueRef hidValueRef)
 	
 	if (logicalMin == 0 && logicalMax == 1)
 	{
-		inputList.push_back(InputAttributesOfHIDValue(hidValueRef, NULL, NULL, nil));
+		inputList.push_back(InputAttributesOfHIDValue(hidValueRef, NULL, NULL, NULL));
 	}
 	else
 	{
 		NSInteger lowerThreshold = ((logicalMax - logicalMin) / 3) + logicalMin;
 		NSInteger upperThreshold = (((logicalMax - logicalMin) * 2) / 3) + logicalMin;
-		NSNumber *onState = [NSNumber numberWithBool:YES];
-		NSNumber *offState = [NSNumber numberWithBool:NO];
+		bool onState = true;
+		bool offState = false;
 		
 		char elementCodeLowerThresholdBuf[256] = {0};
 		char elementCodeUpperThresholdBuf[256] = {0};
@@ -364,18 +364,18 @@ InputAttributesList InputListFromHIDValue(IOHIDValueRef hidValueRef)
 		
 		if (logicalValue <= lowerThreshold)
 		{
-			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeLowerThresholdBuf, NULL, onState));
-			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeUpperThresholdBuf, NULL, offState));
+			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeLowerThresholdBuf, NULL, &onState));
+			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeUpperThresholdBuf, NULL, &offState));
 		}
 		else if (logicalValue >= upperThreshold)
 		{
-			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeLowerThresholdBuf, NULL, offState));
-			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeUpperThresholdBuf, NULL, onState));
+			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeLowerThresholdBuf, NULL, &offState));
+			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeUpperThresholdBuf, NULL, &onState));
 		}
 		else
 		{
-			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeLowerThresholdBuf, NULL, offState));
-			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeUpperThresholdBuf, NULL, onState));
+			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeLowerThresholdBuf, NULL, &offState));
+			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeUpperThresholdBuf, NULL, &offState));
 		}
 	}
 	
@@ -403,14 +403,13 @@ InputAttributesList InputListFromHatSwitchValue(IOHIDValueRef hidValueRef, bool
 	inputList.resize(8);
 	NSInteger logicalMax = IOHIDElementGetLogicalMax(hidElementRef);
 	NSInteger logicalValue = IOHIDValueGetIntegerValue(hidValueRef);
-	
-	NSNumber *onState = [NSNumber numberWithBool:YES];
-	NSNumber *offState = [NSNumber numberWithBool:NO];
+	bool onState = true;
+	bool offState = false;
 	
 	char elementCodeFourWay[4][256];
-	for (unsigned int i = 0; i < 4; i++)
+	for (size_t i = 0; i < 4; i++)
 	{
-		snprintf(elementCodeFourWay[i], 256, "0x%04lX/0x%04lX/%d-FourDirection", (long)elementUsagePage, (long)elementUsage, i);
+		snprintf(elementCodeFourWay[i], 256, "0x%04lX/0x%04lX/%d-FourDirection", (long)elementUsagePage, (long)elementUsage, (unsigned int)i);
 	}
 	
 	const char *elementNameFourWay[4] = {
@@ -420,9 +419,9 @@ InputAttributesList InputListFromHatSwitchValue(IOHIDValueRef hidValueRef, bool
 		"Hatswitch - Left" };
 	
 	char elementCodeEightWay[8][256];
-	for (unsigned int i = 0; i < 8; i++)
+	for (size_t i = 0; i < 8; i++)
 	{
-		snprintf(elementCodeEightWay[i], 256, "0x%04lX/0x%04lX/%d-EightDirection", (long)elementUsagePage, (long)elementUsage, i);
+		snprintf(elementCodeEightWay[i], 256, "0x%04lX/0x%04lX/%d-EightDirection", (long)elementUsagePage, (long)elementUsage, (unsigned int)i);
 	}
 	
 	const char *elementNameEightWay[8] = {
@@ -437,18 +436,18 @@ InputAttributesList InputListFromHatSwitchValue(IOHIDValueRef hidValueRef, bool
 	
 	if (logicalMax == 3)
 	{
-		for (unsigned int i = 0; i <= (unsigned int)logicalMax; i++)
+		for (size_t i = 0; i <= (size_t)logicalMax; i++)
 		{
-			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeFourWay[i], elementNameFourWay[i], (i == (unsigned int)logicalValue) ? onState : offState));
+			inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeFourWay[i], elementNameFourWay[i], (i == (size_t)logicalValue) ? &onState : &offState));
 		}
 	}
 	else if (logicalMax == 7)
 	{
 		if (useEightDirection)
 		{
-			for (unsigned int i = 0; i <= (unsigned int)logicalMax; i++)
+			for (size_t i = 0; i <= (size_t)logicalMax; i++)
 			{
-				inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[i], elementNameEightWay[i], (i == (unsigned int)logicalValue) ? onState : offState));
+				inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[i], elementNameEightWay[i], (i == (size_t)logicalValue) ? &onState : &offState));
 			}
 		}
 		else
@@ -456,66 +455,66 @@ InputAttributesList InputListFromHatSwitchValue(IOHIDValueRef hidValueRef, bool
 			switch (logicalValue)
 			{
 				case 0:
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], onState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], &onState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], &offState));
 					break;
 					
 				case 1:
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], onState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], onState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], &onState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], &onState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], &offState));
 					break;
 					
 				case 2:
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], onState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], &onState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], &offState));
 					break;
 					
 				case 3:
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], onState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], onState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], &onState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], &onState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], &offState));
 					break;
 					
 				case 4:
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], onState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], &onState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], &offState));
 					break;
 					
 				case 5:
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], onState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], onState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], &onState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], &onState));
 					break;
 					
 				case 6:
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], onState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], &onState));
 					break;
 					
 				case 7:
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], onState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], onState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], &onState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], &onState));
 					break;
 					
 				default:
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], offState));
-					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[0], elementNameEightWay[0], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[2], elementNameEightWay[2], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[4], elementNameEightWay[4], &offState));
+					inputList.push_back(InputAttributesOfHIDValue(hidValueRef, elementCodeEightWay[6], elementNameEightWay[6], &offState));
 					break;
 			}
 		}
@@ -597,9 +596,9 @@ BOOL GetOnStateFromHIDValueRef(IOHIDValueRef hidValueRef)
 	return onState;
 }
 
-unsigned int ClearHIDQueue(const IOHIDQueueRef hidQueue)
+size_t ClearHIDQueue(const IOHIDQueueRef hidQueue)
 {
-	unsigned int hidInputClearCount = 0;
+	size_t hidInputClearCount = 0;
 	
 	if (hidQueue == nil)
 	{
@@ -1106,7 +1105,7 @@ static std::tr1::unordered_map<unsigned short, std::string> keyboardNameTable; /
 	
 	const size_t inputCount = inputList->size();
 	
-	for (unsigned int i = 0; i < inputCount; i++)
+	for (size_t i = 0; i < inputCount; i++)
 	{
 		const InputAttributes &inputAttr = (*inputList)[i];
 		if (inputAttr.state != INPUT_ATTRIBUTE_STATE_ON)
@@ -1202,9 +1201,9 @@ static std::tr1::unordered_map<unsigned short, std::string> keyboardNameTable; /
 - (CommandAttributesList) generateCommandListUsingInputList:(const InputAttributesList *)inputList
 {
 	CommandAttributesList cmdList;
-	size_t inputCount = inputList->size();
+	const size_t inputCount = inputList->size();
 	
-	for (unsigned int i = 0; i < inputCount; i++)
+	for (size_t i = 0; i < inputCount; i++)
 	{
 		const InputAttributes &inputAttr = (*inputList)[i];
 		
@@ -1520,8 +1519,17 @@ static std::tr1::unordered_map<unsigned short, std::string> keyboardNameTable; /
 		return error;
 	}
 	
+	AudioStreamBasicDescription inputFormat;
+	UInt32 propertySize = sizeof(inputFormat);
+	
+	error = ExtAudioFileGetProperty(audioFile, kExtAudioFileProperty_FileDataFormat, &propertySize, &inputFormat);
+	if (error != noErr)
+	{
+		return error;
+	}
+	
 	SInt64 fileLengthFrames = 0;
-	UInt32 propertySize = sizeof(fileLengthFrames);
+	propertySize = sizeof(fileLengthFrames);
 	
 	error = ExtAudioFileGetProperty(audioFile, kExtAudioFileProperty_FileLengthFrames, &propertySize, &fileLengthFrames);
 	if (error != noErr)
@@ -1532,17 +1540,18 @@ static std::tr1::unordered_map<unsigned short, std::string> keyboardNameTable; /
 	// Create a new audio file generator.
 	audioFileGenerators[filePathStr] = AudioSampleBlockGenerator();
 	AudioSampleBlockGenerator &theGenerator = audioFileGenerators[filePathStr];
-	u8 *buffer = theGenerator.allocate(fileLengthFrames);
+	const size_t readSize = 32 * 1024;
+	const size_t bufferSize = (size_t)((double)(outputFormat.mSampleRate / inputFormat.mSampleRate) * (double)fileLengthFrames) + readSize;
+	u8 *buffer = theGenerator.allocate(bufferSize);
 	
 	// Read the audio file and fill the generator's buffer.
-	const size_t convertBufferSize = 32 * 1024;
 	AudioBufferList convertedData;
 	convertedData.mNumberBuffers = 1;
 	convertedData.mBuffers[0].mNumberChannels = outputFormat.mChannelsPerFrame;
-	convertedData.mBuffers[0].mDataByteSize = convertBufferSize;
+	convertedData.mBuffers[0].mDataByteSize = readSize;
 	convertedData.mBuffers[0].mData = buffer;
 	
-	UInt32 readFrames = convertBufferSize;
+	UInt32 readFrames = readSize;
 	while (readFrames > 0)
 	{
 		ExtAudioFileRead(audioFile, &readFrames, &convertedData);
@@ -1555,7 +1564,7 @@ static std::tr1::unordered_map<unsigned short, std::string> keyboardNameTable; /
 	
 	// Convert the audio buffer to 7-bit unsigned PCM.
 	buffer = theGenerator.getBuffer();
-	for (SInt64 i = 0; i < fileLengthFrames; i++)
+	for (SInt64 i = 0; i < bufferSize; i++)
 	{
 		*(buffer+i) >>= 1;
 	}
@@ -1781,8 +1790,8 @@ InputAttributesList InputManagerEncodeHIDQueue(const IOHIDQueueRef hidQueue)
 			hidInputList = InputListFromHIDValue(hidValueRef);
 		}
 		
-		size_t hidInputCount = hidInputList.size();
-		for (unsigned int i = 0; i < hidInputCount; i++)
+		const size_t hidInputCount = hidInputList.size();
+		for (size_t i = 0; i < hidInputCount; i++)
 		{
 			if (hidInputList[i].deviceCode[0] == '\0' || hidInputList[i].elementCode[0] == '\0')
 			{
