@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2011 Roger Manuel
+	Copyright (C) 2011-2012 Roger Manuel
 	Copyright (C) 2013 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
@@ -25,7 +25,7 @@ int scanline_filter_b = 2;
 int scanline_filter_c = 2;
 int scanline_filter_d = 4;
 
-// 
+// Parameters passed to the RunVideoFilterTask() thread task function
 typedef struct
 {
 	void *index;
@@ -42,29 +42,41 @@ static const _VideoFilterParamAttributes _VideoFilterParamAttributesList[] = {
 /********************************************************************************************
 	CLASS CONSTRUCTORS
  ********************************************************************************************/
-VideoFilter::VideoFilter(unsigned int srcWidth = 1,
-						 unsigned int srcHeight = 1,
+VideoFilter::VideoFilter(size_t srcWidth,
+						 size_t srcHeight,
 						 VideoFilterTypeID typeID = VideoFilterTypeID_None,
-						 unsigned int numberThreads = 0)
+						 size_t threadCount = 0)
 {
-	SSurface newSurface = {NULL, srcWidth*2, srcWidth, srcHeight};
+	SSurface newSurface;
+	newSurface.Surface = NULL;
+	newSurface.Pitch = srcWidth*2;
+	newSurface.Width = srcWidth;
+	newSurface.Height = srcHeight;
+	
 	_vfSrcSurface = newSurface;
 	_vfDstSurface = newSurface;
 	
 	_isFilterRunning = false;
 	_vfSrcSurfacePixBuffer = NULL;
-	_vfTypeID = typeID;
 	
-	pthread_mutex_init(&_mutexSrc, NULL);
-	pthread_mutex_init(&_mutexDst, NULL);
-	pthread_mutex_init(&_mutexTypeID, NULL);
-	pthread_mutex_init(&_mutexTypeString, NULL);
-	pthread_cond_init(&_condRunning, NULL);
+	if (typeID < VideoFilterTypeIDCount)
+	{
+		_vfAttributes = VideoFilterAttributesList[typeID];
+	}
+	else
+	{
+		_vfAttributes = VideoFilterAttributesList[VideoFilterTypeID_None];
+	}
+	
+	ThreadLockInit(&_lockSrc);
+	ThreadLockInit(&_lockDst);
+	ThreadLockInit(&_lockAttributes);
+	ThreadCondInit(&_condRunning);
 	
 	// Create all threads
-	_vfThread.resize(numberThreads);
+	_vfThread.resize(threadCount);
 	
-	for (unsigned int i = 0; i < numberThreads; i++)
+	for (size_t i = 0; i < threadCount; i++)
 	{
 		_vfThread[i].param.srcSurface = _vfSrcSurface;
 		_vfThread[i].param.dstSurface = _vfDstSurface;
@@ -83,7 +95,7 @@ VideoFilter::VideoFilter(unsigned int srcWidth = 1,
 VideoFilter::~VideoFilter()
 {
 	// Destroy all threads first
-	for (unsigned int i = 0; i < _vfThread.size(); i++)
+	for (size_t i = 0; i < _vfThread.size(); i++)
 	{
 		_vfThread[i].task->finish();
 		_vfThread[i].task->shutdown();
@@ -94,30 +106,29 @@ VideoFilter::~VideoFilter()
 	_vfThread.clear();
 	
 	// Destroy everything else
-	pthread_mutex_lock(&_mutexSrc);
-	pthread_mutex_lock(&this->_mutexDst);
+	ThreadLockLock(&_lockSrc);
+	ThreadLockLock(&_lockDst);
 	
-	while (this->_isFilterRunning)
+	while (_isFilterRunning)
 	{
-		pthread_cond_wait(&this->_condRunning, &this->_mutexDst);
+		ThreadCondWait(&_condRunning, &_lockDst);
 	}
 	
 	free(_vfDstSurface.Surface);
 	_vfDstSurface.Surface = NULL;
 	
-	pthread_mutex_unlock(&_mutexDst);
+	ThreadLockUnlock(&_lockDst);
 	
 	free(_vfSrcSurfacePixBuffer);
 	_vfSrcSurfacePixBuffer = NULL;
 	_vfSrcSurface.Surface = NULL;
 	
-	pthread_mutex_unlock(&_mutexSrc);
+	ThreadLockUnlock(&_lockSrc);
 	
-	pthread_mutex_destroy(&_mutexSrc);
-	pthread_mutex_destroy(&_mutexDst);
-	pthread_mutex_destroy(&_mutexTypeID);
-	pthread_mutex_destroy(&_mutexTypeString);
-	pthread_cond_destroy(&_condRunning);
+	ThreadLockDestroy(&_lockSrc);
+	ThreadLockDestroy(&_lockDst);
+	ThreadLockDestroy(&_lockAttributes);
+	ThreadCondDestroy(&_condRunning);
 }
 
 /********************************************************************************************
@@ -135,11 +146,11 @@ VideoFilter::~VideoFilter()
 		A bool that reports if the resizing was successful. A value of true means success,
 		while a value of false means failure.
  ********************************************************************************************/
-bool VideoFilter::SetSourceSize(const unsigned int width, const unsigned int height)
+bool VideoFilter::SetSourceSize(const size_t width, const size_t height)
 {
 	bool result = false;
 	
-	pthread_mutex_lock(&this->_mutexSrc);
+	ThreadLockLock(&this->_lockSrc);
 	
 	// Overallocate the source buffer by 8 rows of pixels to account for out-of-bounds
 	// memory reads done by some filters.
@@ -162,7 +173,7 @@ bool VideoFilter::SetSourceSize(const unsigned int width, const unsigned int hei
 	// Update the surfaces on threads.
 	size_t threadCount = this->_vfThread.size();
 	
-	for (unsigned int i = 0; i < threadCount; i++)
+	for (size_t i = 0; i < threadCount; i++)
 	{
 		SSurface &threadSrcSurface = this->_vfThread[i].param.srcSurface;
 		threadSrcSurface = this->_vfSrcSurface;
@@ -175,9 +186,10 @@ bool VideoFilter::SetSourceSize(const unsigned int width, const unsigned int hei
 		}
 	}
 	
-	pthread_mutex_unlock(&this->_mutexSrc);
+	ThreadLockUnlock(&this->_lockSrc);
 	
-	result = this->ChangeFilterByID(this->GetTypeID());
+	const VideoFilterAttributes vfAttr = this->GetAttributes();
+	result = this->ChangeFilterByAttributes(&vfAttr);
 	
 	return result;
 }
@@ -204,7 +216,9 @@ bool VideoFilter::ChangeFilterByID(const VideoFilterTypeID typeID)
 		return result;
 	}
 	
-	return this->ChangeFilterByAttributes(&VideoFilterAttributesList[typeID]);
+	result = this->ChangeFilterByAttributes(&VideoFilterAttributesList[typeID]);
+	
+	return result;
 }
 
 /********************************************************************************************
@@ -228,18 +242,24 @@ bool VideoFilter::ChangeFilterByAttributes(const VideoFilterAttributes *vfAttr)
 		return result;
 	}
 	
-	pthread_mutex_lock(&this->_mutexSrc);
-	const unsigned int srcWidth = this->_vfSrcSurface.Width;
-	const unsigned int srcHeight = this->_vfSrcSurface.Height;
-	pthread_mutex_unlock(&this->_mutexSrc);
+	if (vfAttr->scaleMultiply == 0 || vfAttr->scaleDivide < 1)
+	{
+		return result;
+	}
 	
-	const VideoFilterTypeID typeID = vfAttr->typeID;
-	const unsigned int dstWidth = srcWidth * vfAttr->scaleMultiply / vfAttr->scaleDivide;
-	const unsigned int dstHeight = srcHeight * vfAttr->scaleMultiply / vfAttr->scaleDivide;
-	const char *typeString = vfAttr->typeString;
-	const VideoFilterFunc filterFunction = vfAttr->filterFunction;
+	this->SetAttributes(*vfAttr);
+	VideoFilterAttributes &workingAttributes = this->_vfAttributes;
 	
-	pthread_mutex_lock(&this->_mutexDst);
+	ThreadLockLock(&this->_lockSrc);
+	const size_t srcWidth = this->_vfSrcSurface.Width;
+	const size_t srcHeight = this->_vfSrcSurface.Height;
+	ThreadLockUnlock(&this->_lockSrc);
+	
+	const size_t dstWidth = srcWidth * workingAttributes.scaleMultiply / workingAttributes.scaleDivide;
+	const size_t dstHeight = srcHeight * workingAttributes.scaleMultiply / workingAttributes.scaleDivide;
+	const VideoFilterFunc filterFunction = workingAttributes.filterFunction;
+	
+	ThreadLockLock(&this->_lockDst);
 	
 	uint32_t *newSurfaceBuffer = (uint32_t *)calloc(dstWidth * dstHeight, sizeof(uint32_t));
 	if (newSurfaceBuffer == NULL)
@@ -256,9 +276,9 @@ bool VideoFilter::ChangeFilterByAttributes(const VideoFilterAttributes *vfAttr)
 	this->_vfDstSurface.Surface = (unsigned char *)newSurfaceBuffer;
 	
 	// Update the surfaces on threads.
-	size_t threadCount = this->_vfThread.size();
+	const size_t threadCount = this->_vfThread.size();
 	
-	for (unsigned int i = 0; i < threadCount; i++)
+	for (size_t i = 0; i < threadCount; i++)
 	{
 		SSurface &threadDstSurface = this->_vfThread[i].param.dstSurface;
 		threadDstSurface = this->_vfDstSurface;
@@ -273,10 +293,7 @@ bool VideoFilter::ChangeFilterByAttributes(const VideoFilterAttributes *vfAttr)
 		this->_vfThread[i].param.filterFunction = this->_vfFunc;
 	}
 	
-	pthread_mutex_unlock(&this->_mutexDst);
-	
-	this->SetTypeID(typeID);
-	this->SetTypeString(typeString);
+	ThreadLockUnlock(&this->_lockDst);
 	
 	result = true;
 	
@@ -297,11 +314,10 @@ bool VideoFilter::ChangeFilterByAttributes(const VideoFilterAttributes *vfAttr)
  ********************************************************************************************/
 uint32_t* VideoFilter::RunFilter()
 {
-	pthread_mutex_lock(&this->_mutexSrc);
-	pthread_mutex_lock(&this->_mutexDst);
+	ThreadLockLock(&this->_lockSrc);
+	ThreadLockLock(&this->_lockDst);
 	
 	this->_isFilterRunning = true;
-	uint32_t *destBufPtr = (uint32_t *)this->_vfDstSurface.Surface;
 	
 	if (this->_vfFunc == NULL)
 	{
@@ -309,16 +325,15 @@ uint32_t* VideoFilter::RunFilter()
 	}
 	else
 	{
-		size_t threadCount = this->_vfThread.size();
-		
+		const size_t threadCount = this->_vfThread.size();
 		if (threadCount > 0)
 		{
-			for (unsigned int i = 0; i < threadCount; i++)
+			for (size_t i = 0; i < threadCount; i++)
 			{
 				this->_vfThread[i].task->execute(&RunVideoFilterTask, &this->_vfThread[i].param);
 			}
 			
-			for (unsigned int i = 0; i < threadCount; i++)
+			for (size_t i = 0; i < threadCount; i++)
 			{
 				this->_vfThread[i].task->finish();
 			}
@@ -330,16 +345,16 @@ uint32_t* VideoFilter::RunFilter()
 	}
 	
 	this->_isFilterRunning = false;
-	pthread_cond_signal(&this->_condRunning);
+	ThreadCondSignal(&this->_condRunning);
 	
-	pthread_mutex_unlock(&this->_mutexDst);
-	pthread_mutex_unlock(&this->_mutexSrc);
+	ThreadLockUnlock(&this->_lockDst);
+	ThreadLockUnlock(&this->_lockSrc);
 	
-	return destBufPtr;
+	return (uint32_t *)this->_vfDstSurface.Surface;
 }
 
 /********************************************************************************************
-	RunFilterCustom() - STATIC
+	RunFilterCustomByID() - STATIC
 	
 	Runs the pixels from srcBuffer through the video filter, and then stores the
 	resulting pixels into dstBuffer. This function is useful for when your source or
@@ -366,29 +381,78 @@ uint32_t* VideoFilter::RunFilter()
 	Returns:
 		Nothing.
  ********************************************************************************************/
-void VideoFilter::RunFilterCustom(const uint32_t *__restrict__ srcBuffer, uint32_t *__restrict__ dstBuffer,
-								  const unsigned int srcWidth, const unsigned int srcHeight,
-								  const VideoFilterTypeID typeID)
+void VideoFilter::RunFilterCustomByID(const uint32_t *__restrict srcBuffer, uint32_t *__restrict dstBuffer,
+									  const size_t srcWidth, const size_t srcHeight,
+									  const VideoFilterTypeID typeID)
+{
+	if (typeID >= VideoFilterTypeIDCount)
+	{
+		return;
+	}
+	
+	VideoFilter::RunFilterCustomByAttributes(srcBuffer, dstBuffer, srcWidth, srcHeight, &VideoFilterAttributesList[typeID]);
+}
+
+/********************************************************************************************
+	RunFilterCustomByAttributes() - STATIC
+
+	Runs the pixels from srcBuffer through the video filter, and then stores the
+	resulting pixels into dstBuffer. This function is useful for when your source or
+	destination buffers are already established, or when you want to run a filter once
+	without having to instantiate a new filter object.
+
+	Takes:
+		srcBuffer - A pointer to the source pixel buffer. The caller is responsible
+			for ensuring that this buffer is valid. Also note that certain video filters
+			may do out-of-bounds reads, so the caller is responsible for overallocating
+			this buffer in order to avoid crashing on certain platforms.
+
+		dstBuffer - A pointer to the destination pixel buffer. The caller is responsible
+			for ensuring that this buffer is valid and large enough to store all of the
+			destination pixels.
+
+		srcWidth - The source surface width in pixels.
+
+		srcHeight - The source surface height in pixels.
+
+		vfAttr - The video filter's attributes.
+
+	Returns:
+		Nothing.
+ ********************************************************************************************/
+void VideoFilter::RunFilterCustomByAttributes(const uint32_t *__restrict srcBuffer, uint32_t *__restrict dstBuffer,
+											  const size_t srcWidth, const size_t srcHeight,
+											  const VideoFilterAttributes *vfAttr)
 {
 	// Parameter check
 	if (srcBuffer == NULL ||
 		dstBuffer == NULL ||
-		srcWidth == 0 ||
-		srcHeight == 0 ||
-		typeID >= VideoFilterTypeIDCount)
+		srcWidth < 1 ||
+		srcHeight < 1 ||
+		vfAttr == NULL ||
+		vfAttr->scaleMultiply == 0 ||
+		vfAttr->scaleDivide < 1)
 	{
 		return;
 	}
 	
 	// Get the filter attributes
-	const VideoFilterAttributes *vfAttr = &VideoFilterAttributesList[typeID];
-	const unsigned int dstWidth = srcWidth * vfAttr->scaleMultiply / vfAttr->scaleDivide;
-	const unsigned int dstHeight = srcHeight * vfAttr->scaleMultiply / vfAttr->scaleDivide;
+	const size_t dstWidth = srcWidth * vfAttr->scaleMultiply / vfAttr->scaleDivide;
+	const size_t dstHeight = srcHeight * vfAttr->scaleMultiply / vfAttr->scaleDivide;
 	const VideoFilterFunc filterFunction = vfAttr->filterFunction;
 	
 	// Assign the surfaces and run the filter
-	SSurface srcSurface = {(unsigned char *)srcBuffer, srcWidth*2, srcWidth, srcHeight};
-	SSurface dstSurface = {(unsigned char *)dstBuffer, dstWidth*2, dstWidth, dstHeight};
+	SSurface srcSurface;
+	srcSurface.Surface = (unsigned char *)srcBuffer;
+	srcSurface.Pitch = srcWidth*2;
+	srcSurface.Width = srcWidth;
+	srcSurface.Height = srcHeight;
+	
+	SSurface dstSurface;
+	dstSurface.Surface = (unsigned char *)dstBuffer;
+	dstSurface.Pitch = dstWidth*2;
+	dstSurface.Width = dstWidth;
+	dstSurface.Height = dstHeight;
 	
 	if (filterFunction == NULL)
 	{
@@ -426,93 +490,90 @@ const char* VideoFilter::GetTypeStringByID(const VideoFilterTypeID typeID)
 /********************************************************************************************
 	ACCESSORS
  ********************************************************************************************/
+VideoFilterAttributes VideoFilter::GetAttributes()
+{
+	ThreadLockLock(&this->_lockAttributes);
+	VideoFilterAttributes vfAttr = this->_vfAttributes;
+	ThreadLockUnlock(&this->_lockAttributes);
+	
+	return vfAttr;
+}
+
+void VideoFilter::SetAttributes(const VideoFilterAttributes vfAttr)
+{
+	ThreadLockLock(&this->_lockAttributes);
+	this->_vfAttributes = vfAttr;
+	ThreadLockUnlock(&this->_lockAttributes);
+}
+
 VideoFilterTypeID VideoFilter::GetTypeID()
 {
-	pthread_mutex_lock(&this->_mutexTypeID);
-	VideoFilterTypeID typeID = this->_vfTypeID;
-	pthread_mutex_unlock(&this->_mutexTypeID);
+	ThreadLockLock(&this->_lockAttributes);
+	VideoFilterTypeID typeID = this->_vfAttributes.typeID;
+	ThreadLockUnlock(&this->_lockAttributes);
 	
 	return typeID;
 }
 
-void VideoFilter::SetTypeID(const VideoFilterTypeID typeID)
-{
-	pthread_mutex_lock(&this->_mutexTypeID);
-	this->_vfTypeID = typeID;
-	pthread_mutex_unlock(&this->_mutexTypeID);
-}
-
 const char* VideoFilter::GetTypeString()
 {
-	pthread_mutex_lock(&this->_mutexTypeString);
-	const char *typeString = this->_vfTypeString.c_str();
-	pthread_mutex_unlock(&this->_mutexTypeString);
+	ThreadLockLock(&this->_lockAttributes);
+	const char *typeString = this->_vfAttributes.typeString;
+	ThreadLockUnlock(&this->_lockAttributes);
 	
 	return typeString;
 }
 
-void VideoFilter::SetTypeString(const char *typeString)
-{
-	this->SetTypeString(std::string(typeString));
-}
-
-void VideoFilter::SetTypeString(std::string typeString)
-{
-	pthread_mutex_lock(&this->_mutexTypeString);
-	this->_vfTypeString = typeString;
-	pthread_mutex_unlock(&this->_mutexTypeString);
-}
-
 uint32_t* VideoFilter::GetSrcBufferPtr()
 {
-	pthread_mutex_lock(&this->_mutexSrc);
+	ThreadLockLock(&this->_lockSrc);
 	uint32_t *ptr = (uint32_t *)this->_vfSrcSurface.Surface;
-	pthread_mutex_unlock(&this->_mutexSrc);
+	ThreadLockUnlock(&this->_lockSrc);
 	
 	return ptr;
 }
 
 uint32_t* VideoFilter::GetDstBufferPtr()
 {
-	pthread_mutex_lock(&this->_mutexDst);
+	ThreadLockLock(&this->_lockDst);
 	uint32_t *ptr = (uint32_t *)this->_vfDstSurface.Surface;
-	pthread_mutex_unlock(&this->_mutexDst);
+	ThreadLockUnlock(&this->_lockDst);
 	
 	return ptr;
 }
 
-unsigned int VideoFilter::GetSrcWidth()
+size_t VideoFilter::GetSrcWidth()
 {
-	pthread_mutex_lock(&this->_mutexSrc);
-	unsigned int width = this->_vfSrcSurface.Width;
-	pthread_mutex_unlock(&this->_mutexSrc);
+	ThreadLockLock(&this->_lockSrc);
+	size_t width = this->_vfSrcSurface.Width;
+	ThreadLockUnlock(&this->_lockSrc);
 	
 	return width;
 }
 
-unsigned int VideoFilter::GetSrcHeight()
+size_t VideoFilter::GetSrcHeight()
 {
-	pthread_mutex_lock(&this->_mutexSrc);
-	unsigned int height = this->_vfSrcSurface.Height;
-	pthread_mutex_unlock(&this->_mutexSrc);
+	ThreadLockLock(&this->_lockSrc);
+	size_t height = this->_vfSrcSurface.Height;
+	ThreadLockUnlock(&this->_lockSrc);
 	
 	return height;
 }
 
-unsigned int VideoFilter::GetDstWidth()
+size_t VideoFilter::GetDstWidth()
 {
-	pthread_mutex_lock(&this->_mutexDst);
-	unsigned int width = this->_vfDstSurface.Width;
-	pthread_mutex_unlock(&this->_mutexDst);
+	ThreadLockLock(&this->_lockDst);
+	size_t width = this->_vfDstSurface.Width;
+	ThreadLockUnlock(&this->_lockDst);
 	
 	return width;
 }
 
-unsigned int VideoFilter::GetDstHeight()
+size_t VideoFilter::GetDstHeight()
 {
-	pthread_mutex_lock(&this->_mutexDst);
-	unsigned int height = this->_vfDstSurface.Height;
-	pthread_mutex_unlock(&this->_mutexDst);
+	ThreadLockLock(&this->_lockDst);
+	size_t height = this->_vfDstSurface.Height;
+	ThreadLockUnlock(&this->_lockDst);
 	
 	return height;
 }
@@ -526,7 +587,7 @@ int VideoFilter::GetFilterParameteri(VideoFilterParamID paramID)
 {
 	int value = 0;
 	
-	pthread_mutex_lock(&this->_mutexDst);
+	ThreadLockLock(&this->_lockDst);
 	
 	switch (_VideoFilterParamAttributesList[paramID].type)
 	{
@@ -546,7 +607,7 @@ int VideoFilter::GetFilterParameteri(VideoFilterParamID paramID)
 			break;
 	}
 	
-	pthread_mutex_unlock(&this->_mutexDst);
+	ThreadLockUnlock(&this->_lockDst);
 	
 	return value;
 }
@@ -555,7 +616,7 @@ unsigned int VideoFilter::GetFilterParameterui(VideoFilterParamID paramID)
 {
 	unsigned int value = 0;
 	
-	pthread_mutex_lock(&this->_mutexDst);
+	ThreadLockLock(&this->_lockDst);
 	
 	switch (_VideoFilterParamAttributesList[paramID].type)
 	{
@@ -575,7 +636,7 @@ unsigned int VideoFilter::GetFilterParameterui(VideoFilterParamID paramID)
 			break;
 	}
 	
-	pthread_mutex_unlock(&this->_mutexDst);
+	ThreadLockUnlock(&this->_lockDst);
 	
 	return value;
 }
@@ -584,7 +645,7 @@ float VideoFilter::GetFilterParameterf(VideoFilterParamID paramID)
 {
 	float value = 0.0f;
 	
-	pthread_mutex_lock(&this->_mutexDst);
+	ThreadLockLock(&this->_lockDst);
 	
 	switch (_VideoFilterParamAttributesList[paramID].type)
 	{
@@ -604,7 +665,7 @@ float VideoFilter::GetFilterParameterf(VideoFilterParamID paramID)
 			break;
 	}
 	
-	pthread_mutex_unlock(&this->_mutexDst);
+	ThreadLockUnlock(&this->_lockDst);
 	
 	return value;
 }
@@ -616,7 +677,7 @@ void VideoFilter::SetFilterParameteri(VideoFilterParamID paramID, int value)
 		return;
 	}
 	
-	pthread_mutex_lock(&this->_mutexDst);
+	ThreadLockLock(&this->_lockDst);
 	
 	switch (_VideoFilterParamAttributesList[paramID].type)
 	{
@@ -636,7 +697,7 @@ void VideoFilter::SetFilterParameteri(VideoFilterParamID paramID, int value)
 			break;
 	}
 	
-	pthread_mutex_unlock(&this->_mutexDst);
+	ThreadLockUnlock(&this->_lockDst);
 }
 
 void VideoFilter::SetFilterParameterui(VideoFilterParamID paramID, unsigned int value)
@@ -646,7 +707,7 @@ void VideoFilter::SetFilterParameterui(VideoFilterParamID paramID, unsigned int
 		return;
 	}
 	
-	pthread_mutex_lock(&this->_mutexDst);
+	ThreadLockLock(&this->_lockDst);
 	
 	switch (_VideoFilterParamAttributesList[paramID].type)
 	{
@@ -666,7 +727,7 @@ void VideoFilter::SetFilterParameterui(VideoFilterParamID paramID, unsigned int
 			break;
 	}
 	
-	pthread_mutex_unlock(&this->_mutexDst);
+	ThreadLockUnlock(&this->_lockDst);
 }
 
 void VideoFilter::SetFilterParameterf(VideoFilterParamID paramID, float value)
@@ -676,7 +737,7 @@ void VideoFilter::SetFilterParameterf(VideoFilterParamID paramID, float value)
 		return;
 	}
 	
-	pthread_mutex_lock(&this->_mutexDst);
+	ThreadLockLock(&this->_lockDst);
 	
 	switch (_VideoFilterParamAttributesList[paramID].type)
 	{
@@ -696,7 +757,7 @@ void VideoFilter::SetFilterParameterf(VideoFilterParamID paramID, float value)
 			break;
 	}
 	
-	pthread_mutex_unlock(&this->_mutexDst);
+	ThreadLockUnlock(&this->_lockDst);
 }
 
 // Task function for multithreaded filtering
@@ -708,3 +769,86 @@ static void* RunVideoFilterTask(void *arg)
 	
 	return NULL;
 }
+
+#ifdef HOST_WINDOWS 
+void ThreadLockInit(ThreadLock *theLock)
+{
+	InitializeCriticalSection(theLock);
+}
+
+void ThreadLockDestroy(ThreadLock *theLock)
+{
+	DeleteCriticalSection(theLock);
+}
+
+void ThreadLockLock(ThreadLock *theLock)
+{
+	EnterCriticalSection(theLock);
+}
+
+void ThreadLockUnlock(ThreadLock *theLock)
+{
+	LeaveCriticalSection(theLock);
+}
+
+void ThreadCondInit(ThreadCond *theCondition)
+{
+	*theCondition = CreateEvent(NULL, FALSE, FALSE, NULL);
+}
+
+void ThreadCondDestroy(ThreadCond *theCondition)
+{
+	CloseHandle(*theCondition);
+}
+
+void ThreadCondWait(ThreadCond *theCondition, ThreadLock *conditionLock)
+{
+	WaitForSingleObject(*theCondition, INFINITE);
+}
+
+void ThreadCondSignal(ThreadCond *theCondition)
+{
+	SetEvent(*theCondition);
+}
+
+#else
+void ThreadLockInit(ThreadLock *theLock)
+{
+	pthread_mutex_init(theLock, NULL);
+}
+
+void ThreadLockDestroy(ThreadLock *theLock)
+{
+	pthread_mutex_destroy(theLock);
+}
+
+void ThreadLockLock(ThreadLock *theLock)
+{
+	pthread_mutex_lock(theLock);
+}
+
+void ThreadLockUnlock(ThreadLock *theLock)
+{
+	pthread_mutex_unlock(theLock);
+}
+
+void ThreadCondInit(ThreadCond *theCondition)
+{
+	pthread_cond_init(theCondition, NULL);
+}
+
+void ThreadCondDestroy(ThreadCond *theCondition)
+{
+	pthread_cond_destroy(theCondition);
+}
+
+void ThreadCondWait(ThreadCond *theCondition, ThreadLock *conditionLock)
+{
+	pthread_cond_wait(theCondition, conditionLock);
+}
+
+void ThreadCondSignal(ThreadCond *theCondition)
+{
+	pthread_cond_signal(theCondition);
+}
+#endif
