@@ -51,7 +51,7 @@
 
 #include "commandline.h"
 
-#include "addons.h"
+#include "slot2.h"
 
 #include "filter/videofilter.h"
 
@@ -59,11 +59,12 @@
 #include "gdbstub.h"
 #endif
 
-#ifdef HAVE_LIBOSMESA
+#if defined(HAVE_LIBOSMESA) || defined(HAVE_GL_GLX)
 #include <GL/gl.h>
 #include <GL/glu.h>
 #include "OGLRender.h"
 #include "osmesa_3Demu.h"
+#include "glx_3Demu.h"
 #endif
 
 #include "DeSmuME.xpm"
@@ -461,7 +462,7 @@ NULL
 GPU3DInterface *core3DList[] = {
   &gpu3DNull,
   &gpu3DRasterize
-#if defined(HAVE_LIBOSMESA)
+#if defined(HAVE_LIBOSMESA) || defined(HAVE_GL_GLX)
   ,
   &gpu3Dgl
 #endif
@@ -492,7 +493,7 @@ struct modify_key_ctx {
     u8 key_id;
 };
 
-static u16 Cur_Keypad = 0;
+static u16 keys_latch = 0;
 static u16 gdk_shift_pressed = 0;
 u16 Keypad_Temp[NB_KEYS];
 
@@ -528,8 +529,8 @@ fill_configured_features( class configured_features *config,
     { "3d-engine", 0, 0, G_OPTION_ARG_INT, &config->engine_3d, "Select 3d rendering engine. Available engines:\n"
         "\t\t\t\t  0 = 3d disabled\n"
         "\t\t\t\t  1 = internal rasterizer (default)\n"
-#ifdef HAVE_LIBOSMESA
-        "\t\t\t\t  2 = osmesa opengl\n"
+#if defined(HAVE_LIBOSMESA) || defined(HAVE_GL_GLX)
+        "\t\t\t\t  2 = opengl\n"
 #endif
         ,"ENGINE"},
     { "save-type", 0, 0, G_OPTION_ARG_INT, &config->savetype, "Select savetype from the following:\n"
@@ -572,12 +573,12 @@ fill_configured_features( class configured_features *config,
   }
 
   if (config->engine_3d != 0 && config->engine_3d != 1
-#if defined(HAVE_LIBOSMESA)
+#if defined(HAVE_LIBOSMESA) || defined(HAVE_GL_GLX)
            && config->engine_3d != 2
 #endif
           ) {
     g_printerr("Currently available ENGINES: 0, 1"
-#if defined(HAVE_LIBOSMESA)
+#if defined(HAVE_LIBOSMESA) || defined(HAVE_GL_GLX)
             ", 2"
 #endif
             "\n");
@@ -1475,8 +1476,7 @@ static gint Key_Press(GtkWidget *w, GdkEventKey *e, gpointer data)
   if( (e->state & mask) == 0){
     u16 Key = lookup_key(e->keyval);
     if(Key){
-      ADD_KEY( Cur_Keypad, Key );
-      if(desmume_running()) update_keypad(Cur_Keypad);
+      ADD_KEY( keys_latch, Key );
       return 1;
     }
   }
@@ -1501,8 +1501,7 @@ static gint Key_Release(GtkWidget *w, GdkEventKey *e, gpointer data)
       return 1;
   }
   u16 Key = lookup_key(e->keyval);
-  RM_KEY( Cur_Keypad, Key );
-  if(desmume_running()) update_keypad(Cur_Keypad);
+  RM_KEY( keys_latch, Key );
   return 1;
 
 }
@@ -1973,6 +1972,11 @@ gboolean EmuLoop(gpointer data)
         skipped_frames = 0;
     }
 
+    /* Merge the joystick keys with the keyboard ones */
+    process_joystick_events(&keys_latch);
+    /* Update! */
+    update_keypad(keys_latch);
+
     desmume_cycle();    /* Emule ! */
     for (i = 0; i < Frameskip; i++) {
         NDS_SkipNextFrame();
@@ -2158,28 +2162,54 @@ common_gtk_main( class configured_features *my_config)
         fw_config.language = my_config->firmware_language;
     }
 
+
     //------------------addons----------
     my_config->process_addonCommands();
-    addon_type = NDS_ADDON_NONE;
+
+    int slot2_device_type = NDS_SLOT2_AUTO;
+
     if (my_config->is_cflash_configured)
-        addon_type = NDS_ADDON_CFLASH;
+        slot2_device_type = NDS_SLOT2_CFLASH;
 
     if(my_config->gbaslot_rom != "") {
-        addon_type = NDS_ADDON_GBAGAME;
         strncpy(GBAgameName, my_config->gbaslot_rom.c_str(), MAX_PATH);
+        // Check if the file exists and can be opened
+        FILE * test = fopen(GBAgameName, "rb");
+        if (test) {
+            slot2_device_type = NDS_SLOT2_GBACART;
+            fclose(test);
+        }
     }
 
-    switch (addon_type) {
-    case NDS_ADDON_CFLASH:
-    case NDS_ADDON_RUMBLEPAK:
-    case NDS_ADDON_NONE:
-    case NDS_ADDON_GBAGAME:
-        break;
-    default:
-        addon_type = NDS_ADDON_NONE;
-        break;
-    }
-    addonsChangePak (addon_type);
+	switch (slot2_device_type)
+	{
+		case NDS_SLOT2_NONE:
+			break;
+		case NDS_SLOT2_AUTO:
+			break;
+		case NDS_SLOT2_CFLASH:
+			break;
+		case NDS_SLOT2_RUMBLEPAK:
+			break;
+		case NDS_SLOT2_GBACART:
+			break;
+		case NDS_SLOT2_GUITARGRIP:
+			break;
+		case NDS_SLOT2_EXPMEMORY:
+			break;
+		case NDS_SLOT2_EASYPIANO:
+			break;
+		case NDS_SLOT2_PADDLE:
+			break;
+		case NDS_SLOT2_PASSME:
+			break;
+		default:
+			slot2_device_type = NDS_SLOT2_NONE;
+			break;
+	}
+    
+    slot2_Init();
+    slot2_Change((NDS_SLOT2_TYPE)slot2_device_type);
 
 #ifdef GDB_STUB
     if ( my_config->arm9_gdb_port != 0) {
@@ -2376,9 +2406,15 @@ common_gtk_main( class configured_features *my_config)
     //Set the 3D emulation to use
     unsigned core = my_config->engine_3d;
     // setup the gdk 3D emulation;
+#if defined(HAVE_LIBOSMESA) || defined(HAVE_GL_GLX)
+    if(my_config->engine_3d == 2)
+    {
 #if defined(HAVE_LIBOSMESA)
-    if(my_config->engine_3d == 2){
-        core = init_osmesa_3Demu() ? 2 : GPU3D_NULL;
+        core = init_osmesa_3Demu()
+#elif defined(HAVE_GL_GLX)
+        core = init_glx_3Demu()
+#endif
+        ? 2 : GPU3D_NULL;
     }
 #endif
     NDS_3D_ChangeCore(core);
@@ -2428,6 +2464,12 @@ common_gtk_main( class configured_features *my_config)
 
     desmume_free();
 
+#if defined(HAVE_LIBOSMESA)
+    deinit_osmesa_3Demu();
+#elif defined(HAVE_GL_GLX)
+    deinit_glx_3Demu();
+#endif
+
     if ( !gtk_fps_limiter_disabled) {
         /* tidy up the FPS limiter timer and semaphore */
         SDL_RemoveTimer( limiter_timer);
