@@ -17,8 +17,8 @@
 	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#ifndef __FW_H__
-#define __FW_H__
+#ifndef __MC_H__
+#define __MC_H__
 
 #include <stdio.h>
 #include <vector>
@@ -50,94 +50,52 @@
 #define MC_SIZE_256MBITS                0x2000000
 #define MC_SIZE_512MBITS                0x4000000
 
-// ============================================= ADVANsCEne
-#define _ADVANsCEne_BASE_ID "DeSmuME database (ADVANsCEne)\0x1A"
-#define _ADVANsCEne_BASE_VERSION_MAJOR 1
-#define _ADVANsCEne_BASE_VERSION_MINOR 0
-#define _ADVANsCEne_BASE_NAME "ADVANsCEne Nintendo DS Collection"
-class ADVANsCEne
-{
-private:
-	char			database_path[MAX_PATH];		// DeSmuME save types 
-	u8				versionBase[2];
-	char			version[4];
-	time_t			createTime;
-	u8				saveType;
-	u32				crc32;
-	bool			loaded;
-
-	// XML
-	const char		*datName;
-	const char		*datVersion;
-	const char		*urlVersion;
-	const char		*urlDat;
-	bool getXMLConfig(const char *in_filaname);
-public:
-	ADVANsCEne() :	saveType(0xFF),
-					crc32(0),
-					loaded(false)
-	{
-		memset(database_path, 0, sizeof(database_path));
-		memset(versionBase, 0, sizeof(versionBase));
-		memset(version, 0, sizeof(version));
-	}
-	void setDatabase(const char *path) { loaded = false; strcpy(database_path, path); }
-	u32 convertDB(const char *in_filaname);
-	u8 checkDB(const char *serial);
-	u32 getSaveType() { return saveType; }
-	u32 getCRC32() { return crc32; }
-	bool isLoaded() { return loaded; }
-};
-
-
-struct memory_chip_t
-{
-	u8 com;	//persistent command actually handled
-	u32 addr;        //current address for reading/writing
-	u8 addr_shift;   //shift for address (since addresses are transfered by 3 bytes units)
-	u8 addr_size;    //size of addr when writing/reading
-
-	BOOL write_enable;	//is write enabled ?
-
-	u8 *data;       //memory data
-	u32 size;       //memory size
-	BOOL writeable_buffer;	//is "data" writeable ?
-	int type; //type of Memory
-	char *filename;
-	FILE *fp;
-	u8 autodetectbuf[32768];
-	int autodetectsize;
-	
-	// needs only for firmware
-	bool isFirmware;
-	char userfile[MAX_PATH];
-};
-
-//the new backup system by zeromus
+//This "backup device" represents a typical retail NDS save memory accessible via AUXSPI.
+//It is managed as a core emulator service for historical reasons which are bad,
+//and possible infrastructural simplification reasons which are good.
+//Slot-1 devices will map their AUXSPI accesses through to the core-managed BackupDevice to access it for the running software.
 class BackupDevice
 {
 public:
 	BackupDevice();
+	~BackupDevice();
 
-	//signals the save system that we are in our regular mode, loading up a rom. initializes for that case.
-	void load_rom(const char* filename);
 	//signals the save system that we are in MOVIE mode. doesnt load up a rom, and never saves it. initializes for that case.
 	void movie_mode();
-
 	void reset();
 	void close_rom();
 	void forceManualBackupType();
 	void reset_hardware();
 	std::string getFilename() { return filename; }
+
+	u8  readByte(u32 addr, const u8 init);
+	u16 readWord(u32 addr, const u16 init);
+	u32 readLong(u32 addr, const u32 init);
+
+	u8  readByte(const u8 init);
+	u16 readWord(const u16 init);
+	u32 readLong(const u32 init);
+
+	void writeByte(u32 addr, u8  val);
+	void writeWord(u32 addr, u16 val);
+	void writeLong(u32 addr, u32 val);
+
+	void writeByte(u8  val);
+	void writeWord(u16 val);
+	void writeLong(u32 val);
+
+	void seek(u32 pos) { fpMC->fseek(pos, SEEK_SET); }
+
+	void flushBackup() { fpMC->fflush(); }
+	
 	u8 searchFileSaveType(u32 size);
 
 	bool save_state(EMUFILE* os);
 	bool load_state(EMUFILE* is);
 	
 	//commands from mmu
-	void reset_command();
-	u8 data_command(u8,int);
-	std::vector<u8> data;
+	void reset_command() { reset_command_state = true; };
+	u8 data_command(u8, u8);
 
 	//this info was saved before the last reset (used for savestate compatibility)
 	struct SavedInfo
@@ -145,6 +103,9 @@ public:
 		u32 addr_size;
 	} savedInfo;
 
+	void ensure(u32 addr, EMUFILE_FILE *fpOut = NULL);
+	void ensure(u32 addr, u8 val, EMUFILE_FILE *fpOut = NULL);
+
 	//and these are used by old savestates
 	void load_old_state(u32 addr_size, u8* data, u32 datasize);
 	static u32 addr_size_for_old_save_size(int bupmem_size);
@@ -153,36 +114,48 @@ public:
 	static u32 pad_up_size(u32 startSize);
 	void raw_applyUserSettings(u32& size, bool manual = false);
 
-	bool load_duc(const char* filename, u32 force_size = 0);
-	bool load_no_gba(const char *fname, u32 force_size = 0);
-	bool save_no_gba(const char* fname);
-	bool load_raw(const char* filename, u32 force_size = 0);
-	bool save_raw(const char* filename);
-	bool load_movie(EMUFILE* is);
+	u32 trim(void *buf, u32 size);
+	u32 fillLeft(u32 size);
+
 	u32 get_save_duc_size(const char* filename);
 	u32 get_save_nogba_size(const char* filename);
+	u32 get_save_nogba_size(u8 *data);
 	u32 get_save_raw_size(const char* filename);
-
-	//call me once a second or so to lazy flush the save data
-	//here's the reason for this system: we want to dump save files when theyre READ
-	//so that we have a better idea earlier on how large they are. but it slows things down
-	//way too much if we flush whenever we read.
-	void lazy_flush();
-	void flush();
+	bool import_duc(const char* filename, u32 force_size = 0);
+	bool import_no_gba(const char *fname, u32 force_size = 0);
+	bool import_raw(const char* filename, u32 force_size = 0);
+	bool export_no_gba(const char* fname);
+	bool export_raw(const char* filename);
+	bool no_gba_unpack(u8 *&buf, u32 &size);
+	
+	bool load_movie(EMUFILE* is);
 
 	struct {
 			u32 size,padSize,type,addr_size,mem_size;
 		} info;
 
 	bool isMovieMode;
+
+	u32 importDataSize(const char *filename);
+	bool importData(const char *filename, u32 force_size = 0);
+	bool exportData(const char *filename);
+
 private:
+	EMUFILE_FILE *fpMC;
 	std::string filename;
+	u32	fsize;
+	int readFooter();
+	bool write(u8 val);
+	u8	read();
+	bool saveBuffer(u8 *data, u32 size, bool _rewind, bool _truncate = false);
 	
-	bool write_enable;	//is write enabled?
+	bool write_enable;
+	bool reset_command_state;
 	u32 com;	//persistent command actually handled
 	u32 addr_size, addr_counter;
 	u32 addr;
-
+	u8 write_protect;
+	
 	std::vector<u8> data_autodetect;
 	enum STATE {
 		DETECTING = 0, RUNNING = 1
@@ -201,26 +174,12 @@ private:
 	};
 	u8 motionInitState, motionFlag;
 
-	void loadfile();
-	bool _loadfile(const char *fname);
-	void ensure(u32 addr);
-
-	bool flushPending, lazyFlushPending;
-
-private:
-	void resize(u32 size);
+	void checkReset();
+	void detect();
 };
 
-#define NDS_FW_SIZE_V1 (256 * 1024)		/* size of fw memory on nds v1 */
-#define NDS_FW_SIZE_V2 (512 * 1024)		/* size of fw memory on nds v2 */
 
-void mc_init(memory_chip_t *mc, int type);    /* reset and init values for memory struct */
-u8 *mc_alloc(memory_chip_t *mc, u32 size);  /* alloc mc memory */
-void mc_realloc(memory_chip_t *mc, int type, u32 size);      /* realloc mc memory */
-void mc_load_file(memory_chip_t *mc, const char* filename); /* load save file and setup fp */
-void mc_free(memory_chip_t *mc);    /* delete mc memory */
-void fw_reset_com(memory_chip_t *mc);       /* reset communication with mc */
-u8 fw_transfer(memory_chip_t *mc, u8 data);
+
 
 void backup_setManualBackupType(int type);
 void backup_forceManualBackupType();
@@ -230,6 +189,7 @@ struct SAVE_TYPE
 	const char* descr;
 	int media_type;
 	int size;
+	int addr_size;
 };
 
 extern const SAVE_TYPE save_types[];
