@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2009-2011 DeSmuME team
+	Copyright (C) 2009-2013 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -17,37 +17,35 @@
 
 #include <string>
 
-#ifdef _MSC_VER
+#ifdef HOST_WINDOWS 
 #define mkdir _mkdir
 #endif
 
-#if defined(_WINDOWS)
+#if defined(HOST_WINDOWS)
 #include <winsock2.h>
 #include <windows.h>
 #include <direct.h>
 #include "winutil.h"
 #include "common.h"
-#if !defined(WXPORT)
 #include "resource.h"
-#else
-#include <glib.h>
-#endif /* !WXPORT */
 #elif !defined(DESMUME_COCOA)
 #include <glib.h>
-#endif /* _WINDOWS */
+#endif /* HOST_WINDOWS */
 
 #include "time.h"
 #include "utils/xstring.h"
 
-#ifdef _WINDOWS
+#ifdef HOST_WINDOWS
 #define FILE_EXT_DELIMITER_CHAR		'.'
 #define DIRECTORY_DELIMITER_CHAR	'\\'
+#define ALL_DIRECTORY_DELIMITER_STRING "/\\"
 #else
 #define FILE_EXT_DELIMITER_CHAR		'.'
 #define DIRECTORY_DELIMITER_CHAR	'/'
+#define ALL_DIRECTORY_DELIMITER_STRING "/"
 #endif
 
-#ifdef _WINDOWS
+#ifdef HOST_WINDOWS
 void FCEUD_MakePathDirs(const char *fname);
 #endif
 
@@ -57,6 +55,7 @@ public:
 	static bool IsPathRooted (const std::string &path);
 	static std::string GetFileDirectoryPath(std::string filePath);
 	static std::string GetFileNameFromPath(std::string filePath);
+	static std::string ScrubInvalid(std::string str);
 	static std::string GetFileNameWithoutExt(std::string fileName);
 	static std::string GetFileNameFromPathWithoutExt(std::string filePath);
 	static std::string GetFileExt(std::string fileName);
@@ -87,6 +86,7 @@ public:
 	#define NEEDSSAVINGKEY	"needsSaving"
 	#define LASTVISITKEY	"lastVisit"
 	#define LUAKEY			"Lua"
+	#define SLOT1DKEY		"Slot1D"
 	char screenshotFormat[MAX_FORMAT];
 	bool savelastromvisit;
 
@@ -102,6 +102,7 @@ public:
 		SOUNDS,
 		FIRMWARE,
 		MODULE,
+		SLOT1D,
 		MAXKNOWNPATH = MODULE
 	};
 
@@ -115,9 +116,10 @@ public:
 	char pathToFirmware[MAX_PATH];
 	char pathToModule[MAX_PATH];
 	char pathToLua[MAX_PATH];
+	char pathToSlot1D[MAX_PATH];
 
-	void init(const char *filename) {
-
+	void init(const char *filename) 
+	{
 		path = std::string(filename);
 
 		//extract the internal part of the logical rom name
@@ -132,7 +134,7 @@ public:
 
 	void LoadModulePath()
 	{
-#if defined(_WINDOWS)
+#if defined(HOST_WINDOWS)
 
 		char *p;
 		ZeroMemory(pathToModule, sizeof(pathToModule));
@@ -141,6 +143,12 @@ public:
 		p = pathToModule + lstrlen(pathToModule);
 		while (p >= pathToModule && *p != DIRECTORY_DELIMITER_CHAR) p--;
 		if (++p >= pathToModule) *p = 0;
+
+		extern char* _hack_alternateModulePath;
+		if(_hack_alternateModulePath)
+		{
+			strcpy(pathToModule,_hack_alternateModulePath);
+		}
 #elif defined(DESMUME_COCOA)
 		std::string pathStr = Path::GetFileDirectoryPath(path);
 
@@ -161,7 +169,7 @@ public:
 
 	void GetDefaultPath(char *pathToDefault, const char *key, int maxCount)
 	{
-#ifdef _WINDOWS
+#ifdef HOST_WINDOWS
 		std::string temp = (std::string)"." + DIRECTORY_DELIMITER_CHAR + pathToDefault;
 		strncpy(pathToDefault, temp.c_str(), maxCount);
 #else
@@ -171,7 +179,7 @@ public:
 
 	void ReadKey(char *pathToRead, const char *key)
 	{
-#ifdef _WINDOWS
+#ifdef HOST_WINDOWS
 		GetPrivateProfileString(SECTION, key, key, pathToRead, MAX_PATH, IniName);
 		if(strcmp(pathToRead, key) == 0) {
 			//since the variables are all intialized in this file they all use MAX_PATH
@@ -197,11 +205,17 @@ public:
 		ReadKey(pathToSounds, SOUNDKEY);
 		ReadKey(pathToFirmware, FIRMWAREKEY);
 		ReadKey(pathToLua, LUAKEY);
-#ifdef _WINDOWS
+		ReadKey(pathToSlot1D, SLOT1DKEY);
+#ifdef HOST_WINDOWS
 		GetPrivateProfileString(SECTION, FORMATKEY, "%f_%s_%r", screenshotFormat, MAX_FORMAT, IniName);
 		savelastromvisit	= GetPrivateProfileBool(SECTION, LASTVISITKEY, true, IniName);
 		currentimageformat	= (ImageFormat)GetPrivateProfileInt(SECTION, DEFAULTFORMATKEY, PNG, IniName);
 		r4Format = (R4Format)GetPrivateProfileInt(SECTION, R4FORMATKEY, R4_CHEAT_DAT, IniName);
+		if ((r4Format != R4_CHEAT_DAT) && (r4Format != R4_USRCHEAT_DAT))
+		{
+			r4Format = R4_USRCHEAT_DAT;
+			WritePrivateProfileInt(SECTION, R4FORMATKEY, r4Format, IniName);
+		}
 #endif
 	/*
 		needsSaving		= GetPrivateProfileInt(SECTION, NEEDSSAVINGKEY, TRUE, IniName);
@@ -244,6 +258,9 @@ public:
 		case MODULE:
 			pathToCopy = pathToModule;
 			break;
+		case SLOT1D:
+			pathToCopy = pathToSlot1D;
+			break;
 		}
 
 		if(action == GET)
@@ -265,7 +282,7 @@ public:
 			}
 
 			strncpy(buffer, thePath.c_str(), MAX_PATH);
-			#ifdef _WINDOWS
+			#ifdef HOST_WINDOWS
 			FCEUD_MakePathDirs(buffer);
 			#endif
 		}
@@ -380,25 +397,15 @@ public:
 
 	enum R4Format
 	{
-#if defined(_WINDOWS) && !defined(WXPORT)
-		R4_CHEAT_DAT = IDC_R4TYPE1,
-		R4_USRCHEAT_DAT = IDC_R4TYPE2
-#else
-		R4_CHEAT_DAT,
-		R4_USRCHEAT_DAT
-#endif
+		R4_CHEAT_DAT = 0,
+		R4_USRCHEAT_DAT = 1
 	};
 	R4Format r4Format;
 
 	enum ImageFormat
 	{
-#if defined(_WINDOWS) && !defined(WXPORT)
-		PNG = IDC_PNG,
-		BMP = IDC_BMP
-#else
-		PNG,
-		BMP
-#endif
+		PNG = 0,
+		BMP = 1
 	};
 
 	ImageFormat currentimageformat;
@@ -412,6 +419,7 @@ public:
 		std::string romPath = filename;
 
 		RomName = Path::GetFileNameFromPath(romPath);
+		RomName = Path::ScrubInvalid(RomName);
 		RomDirectory = Path::GetFileDirectoryPath(romPath);
 	}
 
@@ -422,6 +430,8 @@ public:
 
 	std::string GetRomNameWithoutExtension()
 	{
+		if (RomName.c_str() == NULL)
+			return "";
 		return Path::GetFileNameWithoutExt(RomName);
 	}
 
