@@ -33,7 +33,7 @@
 #include <math.h>
 #include <string.h>
 
-#ifndef _MSC_VER
+#ifndef _MSC_VER 
 #include <stdint.h>
 #endif
 
@@ -1099,9 +1099,11 @@ static char SoftRastInit(void)
 		_HACK_viewer_rasterizerUnit.SLI_VALUE = 0;
 
 		rasterizerCores = CommonSettings.num_cores;
+
 		if (rasterizerCores > _MAX_CORES) 
 			rasterizerCores = _MAX_CORES;
-		if(CommonSettings.num_cores <= 1)
+
+		if(CommonSettings.num_cores == 1)
 		{
 			rasterizerCores = 1;
 			rasterizerUnit[0].SLI_MASK = 0;
@@ -1368,8 +1370,10 @@ void SoftRasterizerEngine::framebufferProcess()
 			edgeMarkColors[i].g = GFX3D_5TO6(edgeMarkColors[i].g);
 			edgeMarkColors[i].b = GFX3D_5TO6(edgeMarkColors[i].b);
 
-			// this seems to be the only thing that selectively disables edge marking
-			edgeMarkDisabled[i] = (col == 0x7FFF);
+			//zero 20-jun-2013 - this doesnt make any sense. at least, it should be related to the 0x8000 bit. if this is undocumented behaviour, lets write about which scenario proves it here, or which scenario is requiring this code.
+			//// this seems to be the only thing that selectively disables edge marking
+			//edgeMarkDisabled[i] = (col == 0x7FFF);
+			edgeMarkDisabled[i] = 0;
 		}
 
 		for(int i=0,y=0;y<192;y++)
