@@ -18,8 +18,6 @@
 	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-//MUST SAVE ADDONS! OMG HOW DID WE FORGET THAT
-
 #ifdef HAVE_LIBZ
 #include <zlib.h>
 #endif
@@ -43,10 +41,12 @@
 #include "movie.h"
 #include "mic.h"
 #include "MMU_timing.h"
+#include "slot1.h"
+#include "slot2.h"
 
 #include "path.h"
 
-#ifdef _WINDOWS
+#ifdef HOST_WINDOWS
 #include "windows/main.h"
 #endif
 
@@ -56,6 +56,12 @@ int lastSaveState = 0;		//Keeps track of last savestate used for quick save/load
 //since this isnt supported right now, it is declared in here to make things compile
 #define SS_INDIRECT            0x80000000
 
+u32 _DESMUME_version = EMU_DESMUME_VERSION_NUMERIC();
+u32 svn_rev = EMU_DESMUME_SUBVERSION_NUMERIC();
+s64 save_time = 0;
+NDS_SLOT1_TYPE slot1Type = NDS_SLOT1_RETAIL_AUTO;
+NDS_SLOT2_TYPE slot2Type = NDS_SLOT2_AUTO;
+
 savestates_t savestates[NB_STATES];
 
 #define SAVESTATE_VERSION       12
@@ -64,6 +70,17 @@ static const char* magic = "DeSmuME SState\0";
 //a savestate chunk loader can set this if it wants to permit a silent failure (for compatibility)
 static bool SAV_silent_fail_flag;
 
+SFORMAT SF_NDS_INFO[]={
+	{ "GINF", 1, sizeof(gameInfo.header), &gameInfo.header},
+	{ "GRSZ", 1, 4, &gameInfo.romsize},
+	{ "DVMJ", 1, 1, (void*)&DESMUME_VERSION_MAJOR},
+	{ "DVMI", 1, 1, (void*)&DESMUME_VERSION_MINOR},
+	{ "DSBD", 1, 1, (void*)&DESMUME_VERSION_BUILD},
+	{ "GREV", 1, 4, &svn_rev},
+	{ "GTIM", 1, 8, &save_time},
+	{ 0 }
+};
+
 SFORMAT SF_ARM7[]={
 	{ "7INS", 4, 1, &NDS_ARM7.instruction },
 	{ "7INA", 4, 1, &NDS_ARM7.instruct_adr },
@@ -170,6 +187,7 @@ SFORMAT SF_NDS[]={
 	{ "_STX", 2, 1, &nds.scr_touchX},
 	{ "_STY", 2, 1, &nds.scr_touchY},
 	{ "_TPB", 4, 1, &nds.isTouch},
+	{ "_IFB", 4, 1, &nds.isFakeBooted},
 	{ "_DBG", 4, 1, &nds._DebugConsole},
 	{ "_ENS", 4, 1, &nds.ensataEmulation},
 	{ "_TYP", 4, 1, &nds.ConsoleType},
@@ -214,7 +232,7 @@ SFORMAT SF_MMU[]={
 	{ "M_SX", 1, 2,       &MMU.SPI_CNT},
 	{ "M_SC", 1, 2,       &MMU.SPI_CMD},
 	{ "MASX", 1, 2,       &MMU.AUX_SPI_CNT},
-	{ "MASC", 1, 2,       &MMU.AUX_SPI_CMD},
+	//{ "MASC", 1, 2,       &MMU.AUX_SPI_CMD}, //zero 20-aug-2013 - this seems pointless
 
 	{ "MWRA", 1, 2,       &MMU.WRAMCNT},
 
@@ -236,10 +254,11 @@ SFORMAT SF_MMU[]={
 	{ "BUWR", 4, 1,       &MMU.fw.writeable_buffer},
 	//end memory chips
 
-	{ "MC0A", 4, 1,       &MMU.dscard[0].address},
-	{ "MC0T", 4, 1,       &MMU.dscard[0].transfer_count},
-	{ "MC1A", 4, 1,       &MMU.dscard[1].address},
-	{ "MC1T", 4, 1,       &MMU.dscard[1].transfer_count},
+	//TODO:slot-1 plugins
+	{ "GC0T", 4, 1,       &MMU.dscard[0].transfer_count},
+	{ "GC0M", 4, 1,       &MMU.dscard[0].mode},
+	{ "GC1T", 4, 1,       &MMU.dscard[1].transfer_count},
+	{ "GC1M", 4, 1,       &MMU.dscard[1].mode},
 	//{ "MCHT", 4, 1,       &MMU.CheckTimers},
 	//{ "MCHD", 4, 1,       &MMU.CheckDMAs},
 
@@ -272,39 +291,6 @@ SFORMAT SF_MOVIE[]={
 	{ 0 }
 };
 
-static void mmu_savestate(EMUFILE* os)
-{
-	u32 version = 8;
-	write32le(version,os);
-	
-	//version 2:
-	MMU_new.backupDevice.save_state(os);
-	
-	//version 3:
-	MMU_new.gxstat.savestate(os);
-	for(int i=0;i<2;i++)
-		for(int j=0;j<4;j++)
-			MMU_new.dma[i][j].savestate(os);
-
-	MMU_timing.arm9codeFetch.savestate(os, version);
-	MMU_timing.arm9dataFetch.savestate(os, version);
-	MMU_timing.arm7codeFetch.savestate(os, version);
-	MMU_timing.arm7dataFetch.savestate(os, version);
-	MMU_timing.arm9codeCache.savestate(os, version);
-	MMU_timing.arm9dataCache.savestate(os, version);
-
-	//version 4:
-	MMU_new.sqrt.savestate(os);
-	MMU_new.div.savestate(os);
-
-	//version 6:
-	MMU_new.dsi_tsc.save_state(os);
-
-	//version 8:
-	os->write32le(MMU.fw.size);
-	os->fwrite(MMU.fw.data,MMU.fw.size);
-}
-
 // TODO: integrate the new wifi state variables once everything is settled
 SFORMAT SF_WIFI[]={
 	{ "W000", 4, 1, &wifiMac.powerOn},
@@ -382,6 +368,119 @@ SFORMAT SF_WIFI[]={
 	{ 0 }
 };
 
+extern SFORMAT SF_RTC[];
+
+static u8 reserveVal = 0;
+SFORMAT reserveChunks[] = {
+	{ "RESV", 1, 1, &reserveVal},
+	{ 0 }
+};
+
+static bool s_slot1_loadstate(EMUFILE* is, int size)
+{
+	u32 version = is->read32le();
+
+	//version 0:
+	if(version >= 0)
+	{
+		u8 slotID = is->read32le();
+		slot1Type = NDS_SLOT1_RETAIL_AUTO;
+		if (version >= 1)
+			slot1_getTypeByID(slotID, slot1Type);
+
+		slot1_Change(slot1Type);
+
+		EMUFILE_MEMORY temp;
+		is->readMemoryStream(&temp);
+		temp.fseek(0,SEEK_SET);
+		slot1_Loadstate(&temp);
+	}
+
+	return true;
+}
+
+static void s_slot1_savestate(EMUFILE* os)
+{
+	u32 version = 1;
+	os->write32le(version);
+
+	u8 slotID = (u8)slot1_List[slot1_GetSelectedType()]->info()->id();
+	os->write32le(slotID);
+
+	EMUFILE_MEMORY temp;
+	slot1_Savestate(&temp);
+	os->writeMemoryStream(&temp);
+}
+
+static bool s_slot2_loadstate(EMUFILE* is, int size)
+{
+	u32 version = is->read32le();
+
+	//version 0:
+	if(version >= 0)
+	{
+		slot2Type = NDS_SLOT2_AUTO;
+		u8 slotID = is->read32le();
+		if (version == 0)
+			slot2_getTypeByID(slotID, slot2Type);
+		slot2_Change(slot2Type);
+
+		EMUFILE_MEMORY temp;
+		is->readMemoryStream(&temp);
+		temp.fseek(0,SEEK_SET);
+		slot2_Loadstate(&temp);
+	}
+
+	return true;
+}
+
+static void s_slot2_savestate(EMUFILE* os)
+{
+	u32 version = 0;
+	os->write32le(version);
+
+	//version 0:
+	u8 slotID = (u8)slot2_List[slot2_GetSelectedType()]->info()->id();
+	os->write32le(slotID);
+
+	EMUFILE_MEMORY temp;
+	slot2_Savestate(&temp);
+	os->writeMemoryStream(&temp);
+}
+
+static void mmu_savestate(EMUFILE* os)
+{
+	u32 version = 8;
+	write32le(version,os);
+	
+	//version 2:
+	MMU_new.backupDevice.save_state(os);
+	
+	//version 3:
+	MMU_new.gxstat.savestate(os);
+	for(int i=0;i<2;i++)
+		for(int j=0;j<4;j++)
+			MMU_new.dma[i][j].savestate(os);
+
+	MMU_timing.arm9codeFetch.savestate(os, version);
+	MMU_timing.arm9dataFetch.savestate(os, version);
+	MMU_timing.arm7codeFetch.savestate(os, version);
+	MMU_timing.arm7dataFetch.savestate(os, version);
+	MMU_timing.arm9codeCache.savestate(os, version);
+	MMU_timing.arm9dataCache.savestate(os, version);
+
+	//version 4:
+	MMU_new.sqrt.savestate(os);
+	MMU_new.div.savestate(os);
+
+	//version 6:
+	MMU_new.dsi_tsc.save_state(os);
+
+	//version 8:
+	os->write32le(MMU.fw.size);
+	os->fwrite(MMU.fw.data,MMU.fw.size);
+}
+
 static bool mmu_loadstate(EMUFILE* is, int size)
 {
 	//read version
@@ -918,9 +1017,13 @@ bool savestate_save (const char *file_name)
 	} else return false;
 }
 
-extern SFORMAT SF_RTC[];
-
 static void writechunks(EMUFILE* os) {
+
+	DateTime tm = DateTime::get_Now();
+	svn_rev = EMU_DESMUME_SUBVERSION_NUMERIC();
+
+	save_time = tm.get_Ticks();
+
 	savestate_WriteChunk(os,1,SF_ARM9);
 	savestate_WriteChunk(os,2,SF_ARM7);
 	savestate_WriteChunk(os,3,cp15_savestate);
@@ -938,18 +1041,47 @@ static void writechunks(EMUFILE* os) {
 	savestate_WriteChunk(os,101,mov_savestate);
 	savestate_WriteChunk(os,110,SF_WIFI);
 	savestate_WriteChunk(os,120,SF_RTC);
+	savestate_WriteChunk(os,130,SF_NDS_INFO);
+	savestate_WriteChunk(os,140,s_slot1_savestate);
+	savestate_WriteChunk(os,150,s_slot2_savestate);
+	// reserved for future versions
+	savestate_WriteChunk(os,160,reserveChunks);
+	savestate_WriteChunk(os,170,reserveChunks);
+	savestate_WriteChunk(os,180,reserveChunks);
+	// ============================
 	savestate_WriteChunk(os,0xFFFFFFFF,(SFORMAT*)0);
 }
 
 static bool ReadStateChunks(EMUFILE* is, s32 totalsize)
 {
 	bool ret = true;
+	bool haveInfo = false;
+	
+	s64 save_time = 0;
+	u32 romsize = 0;
+	u8 version_major = 0;
+	u8 version_minor = 0;
+	u8 version_build = 0;
+	
+	NDS_header header;
+	SFORMAT SF_INFO[]={
+		{ "GINF", 1, sizeof(header), &header},
+		{ "GRSZ", 1, 4, &romsize},
+		{ "DVMJ", 1, 1, &version_major},
+		{ "DVMI", 1, 1, &version_minor},
+		{ "DSBD", 1, 1, &version_build},
+		{ "GREV", 1, 4, &svn_rev},
+		{ "GTIM", 1, 8, &save_time},
+		{ 0 }
+	};
+	memset(&header, 0, sizeof(header));
+
 	while(totalsize > 0)
 	{
-		uint32 size;
-		u32 t;
+		u32 size = 0;
+		u32 t = 0;
 		if(!read32le(&t,is))  { ret=false; break; }
-		if(t == 0xFFFFFFFF) goto done;
+		if(t == 0xFFFFFFFF) break;
 		if(!read32le(&size,is))  { ret=false; break; }
 		switch(t)
 		{
@@ -970,14 +1102,56 @@ static bool ReadStateChunks(EMUFILE* is, s32 totalsize)
 			case 101: if(!mov_loadstate(is, size)) ret=false; break;
 			case 110: if(!ReadStateChunk(is,SF_WIFI,size)) ret=false; break;
 			case 120: if(!ReadStateChunk(is,SF_RTC,size)) ret=false; break;
+			case 130: if(!ReadStateChunk(is,SF_INFO,size)) ret=false; else haveInfo=true; break;
+			case 140: if(!s_slot1_loadstate(is, size)) ret=false; break;
+			case 150: if(!s_slot2_loadstate(is, size)) ret=false; break;
+			// reserved for future versions
+			case 160:
+			case 170:
+			case 180:
+				if(!ReadStateChunk(is,reserveChunks,size)) ret=false;
+			break;
+			// ============================
+				
 			default:
-				ret=false;
-				break;
+				return false;
 		}
 		if(!ret)
 			return false;
 	}
-done:
+
+	if (haveInfo)
+	{
+		char buf[14] = {0};
+		memset(&buf[0], 0, sizeof(buf));
+		memcpy(buf, header.gameTile, sizeof(header.gameTile));
+		printf("Savestate info:\n");
+		if (version_major | version_minor | version_build)
+		{
+			char buf[32] = {0};
+			if (svn_rev != 0xFFFFFFFF)
+				sprintf(buf, " svn %u", svn_rev);
+			printf("\tDeSmuME version: %u.%u.%u%s\n", version_major, version_minor, version_build, buf);
+		}
+
+		if (save_time)
+		{
+			static const char *wday[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
+			DateTime tm = save_time;
+			printf("\tSave created: %04d-%03s-%02d %s %02d:%02d:%02d\n", tm.get_Year(), DateTime::GetNameOfMonth(tm.get_Month()), tm.get_Day(), wday[tm.get_DayOfWeek()%7], tm.get_Hour(), tm.get_Minute(), tm.get_Second());
+		}
+		printf("\tGame title: %s\n", buf);
+		printf("\tGame code: %c%c%c%c\n", header.gameCode[3], header.gameCode[2], header.gameCode[1], header.gameCode[0]);
+		printf("\tMaker code: %c%c (0x%04X) - %s\n", header.makerCode & 0xFF, header.makerCode >> 8, header.makerCode, getDeveloperNameByID(header.makerCode).c_str());
+		printf("\tDevice capacity: %dMb (real size %dMb)\n", ((128 * 1024) << header.cardSize) / (1024 * 1024), romsize / (1024 * 1024));
+		printf("\tCRC16: %04Xh\n", header.CRC16);
+		printf("\tHeader CRC16: %04Xh\n", header.headerCRC16);
+		printf("\tSlot1: %s\n", slot1_List[slot1Type]->info()->name());
+		printf("\tSlot2: %s\n", slot2_List[slot2Type]->info()->name());
+
+		if (gameInfo.romsize != romsize || memcmp(&gameInfo.header, &header, sizeof(header)) != 0)
+			msgbox->warn("The savestate you are loading does not match the ROM you are running.\nYou should find the correct ROM");
+	}
 
 	return ret;
 }
@@ -1020,9 +1194,9 @@ bool savestate_load(EMUFILE* is)
 	if(is->fail() || memcmp(header,magic,16))
 		return false;
 
-	u32 ssversion,dversion,len,comprlen;
+	u32 ssversion,len,comprlen;
 	if(!read32le(&ssversion,is)) return false;
-	if(!read32le(&dversion,is)) return false;
+	if(!read32le(&_DESMUME_version,is)) return false;
 	if(!read32le(&len,is)) return false;
 	if(!read32le(&comprlen,is)) return false;
 
@@ -1075,11 +1249,7 @@ bool savestate_load(EMUFILE* is)
 
 	if(!x && !SAV_silent_fail_flag)
 	{
-		printf("Error loading savestate. It failed halfway through;\nSince there is no savestate backup system, your current game session is wrecked");
-#ifdef _WINDOWS
-		//HACK! we really need a better way to handle this kind of feedback
-		MessageBox(0,"Error loading savestate. It failed halfway through;\nSince there is no savestate backup system, your current game session is wrecked",0,0);
-#endif
+		msgbox->error("Error loading savestate. It failed halfway through;\nSince there is no savestate backup system, your current game session is wrecked");
 		return false;
 	}
 
