@@ -21,9 +21,20 @@
 
 //analyze microsoft compilers
 #ifdef _MSC_VER
-#define _WINDOWS
-//todo - everyone will want to support this eventually, i suppose
-#include "config.h"
+	#define HOST_WINDOWS
+	
+	//todo - everyone will want to support this eventually, i suppose
+	#include "config.h"
+
+#endif //_MSC_VER
+
+// Determine CPU architecture for platforms that don't use the autoconf script
+#if defined(HOST_WINDOWS) || defined(DESMUME_COCOA)
+	#if defined(__x86_64__) || defined(__LP64) || defined(__IA64__) || defined(_M_X64) || defined(_WIN64)
+		#define HOST_64
+	#else
+		#define HOST_32
+	#endif
 #endif
 
 //enforce a constraint: gdb stub requires developer
@@ -37,8 +48,7 @@
 #define IF_DEVELOPER(X)
 #endif
 
-#ifdef _WINDOWS
-	//#define HAVE_WX //not useful yet....
+#ifdef HOST_WINDOWS
 	#define HAVE_LIBAGG
 	#define ENABLE_SSE
 	#define ENABLE_SSE2
@@ -83,6 +93,9 @@
 #endif
 
 
+//------------alignment macros-------------
+//dont apply these to types without further testing. it only works portably here on declarations of variables
+//cant we find a pattern other people use more successfully?
 #if defined(_MSC_VER) || defined(__INTEL_COMPILER)
 #define DS_ALIGN(X) __declspec(align(X))
 #elif defined(__GNUC__)
@@ -90,11 +103,10 @@
 #else
 #define DS_ALIGN(X)
 #endif
-
 #define CACHE_ALIGN DS_ALIGN(32)
-
 //use this for example when you want a byte value to be better-aligned
 #define FAST_ALIGN DS_ALIGN(4)
+//---------------------------------------------
 
 #ifdef __MINGW32__
 #define FASTCALL __attribute__((fastcall))
