@@ -24,14 +24,12 @@
 #include "bits.h"
 
 
-#ifdef _WINDOWS
+#ifdef HOST_WINDOWS
 	#include <winsock2.h> 	 
 	#include <ws2tcpip.h>
 	#define socket_t    SOCKET 	 
 	#define sockaddr_t  SOCKADDR
-	#ifndef WXPORT
-		#include "windriver.h"
-	#endif
+	#include "windriver.h"
 	#define PCAP_DEVICE_NAME description
 #else
 	#include <unistd.h> 	 
